/// <reference path="/dev/AIRAliases.js" />

require_("script/http-request.js");
require_("script/google-hr.js");
require_("script/file.js");
require_("script/ui.js");
require_("script/vejis/dom.js");
require_("script/vejis/drop.js");
require_("script/vejis/drop.ui.js");
require_("script/table.js");

use_("http-request", "google-hr", "file", "dom", "enhanced-table", "drop", "drop.ui",
    function(hr, ghr, file, dom, enhancedTable, drop) {
        module_("favorites", function() {

            //refresh favorites
            //var favoritesFile = new air.File("app-storage:/favorites");
            //file.writeText(favoritesFile, "");

            var favoritesMod = this;

            var favoritesTables = {};
            var trsFavoritesMaps = {};

            this.init = function() {
                var favorites = favoritesMod.getFavoritesFromDB();

                //air.trace(JSON.stringify(favorites));

                for (var i in favorites) {
                    var tab = favorites[i];
                    this.initTab(tab);
                }

                $('.jqTooltip').tooltip();

                $("#favorites-tabs").tabs();

                favoritesMod.initAddTabButton();

                if (favorites.length < 2) $(".tabs-close").hide();
                else $(".tabs-close").show();
            };

            this.initTab = function(tab) {
                $("#favorites-tabs>ul").append(
                    '<li tab-id="' + tab.id + '">' +
                    '<a id="tab-button-' + tab.id + '" href="#tabs-left-' + tab.id + '">' +
                    '<span class="tab_name jqTooltip" title="double click to edit">' + tab.name + '</span>' +
                    '<span class="tab_edit" style="display: none;"><input type="text" class="tab_name_input" value="' + tab.name + '"  style="width: 70px; height: 20px;" maxlength="15" /></span>' +
                    '</a>' +
                    '<a href="javascript:void(0)" class="tabs-close" id="tabs-close' + tab.id + '">x</a>' +
                    '</li>'
                );
                $("#favorites-tabs").append('<div id="tabs-left-' + tab.id + '"><div id="favorites-table-outter-' + tab.id + '" class="table-outter"></div></div>');

                $('#tab-button-' + tab.id + ' .tab_name').dblclick(function() {
                    $('#tab-button-' + tab.id + ' .tab_name').hide();
                    $('#tab-button-' + tab.id + ' .tab_edit').show();
                    $('#tab-button-' + tab.id + ' .tab_name_input').select();
                });

                $('#tab-button-' + tab.id + ' .tab_name_input').blur(function() {
                    favoritesMod.saveTabName(tab.id);
                });

                $('#tab-button-' + tab.id + ' .tab_name_input').keypress(function(e) {
                    if (e.keyCode == 13) {
                        favoritesMod.saveTabName(tab.id);
                    }
                });

                $('#tab-button-' + tab.id).click(function() {
                    var tab_id = $(this).parents("li").attr("tab-id");
                    setTimeout(showThisTable, 10);

                    function showThisTable() {
                        favoritesMod.initTable(tab_id);
                    }
                });

                $('#tabs-close' + tab.id).click(function() {
                    var tab_id = $(this).parents("li").attr("tab-id");
                    favoritesMod.closeTab(tab_id);
                });
            };

            var tabsInnitted = [];
            this.initTable = function(tab_id) {
                if (tabsInnitted[tab_id])
                    return false;

                tabsInnitted[tab_id] = true;
                trsFavoritesMaps[tab_id] = [];

                favoritesTables[tab_id] = new enhancedTable.Table(450);
                document.getElementById('favorites-table-outter-' + tab_id).appendChild(favoritesTables[tab_id].element);
                favoritesTables[tab_id].initialize();

                jQuery(window).resize(function() {
                    setHeight();
                });

                $("#favorites-tab").click(function() {
                    setHeight();
                });

                setHeight();

                function setHeight() {
                    if (favoritesTables[tab_id]) {
                        var height = document.getElementById("favorites-wrapper").offsetHeight;
                        favoritesTables[tab_id].setSize(height - 200);
                        $('#favorites-table-outter-' + tab_id + ' table tr td:eq(0)').css('width', '130px');
                        $('#favorites-table-outter-' + tab_id + ' table tr td:eq(6)').css('width', '190px');
                    }
                }

                var wrapper = favoritesTables[tab_id].element;
                var table = wrapper.getElementsByTagName("table")[0];
                var tableWrapper = wrapper.getElementsByClassName("table-wrapper")[0];

                var header = document.createElement("tr");
                header.className = "table-header";

                var headerData = [{
                    name: "keyword_category",
                    items: ["Keyword/Category"],
                    keys: ["keyword_category"],
                    sortable: false
                }, {
                    name: "site",
                    items: ["Site"],
                    keys: ["site"],
                    sortable: false
                }, {
                    name: "delivery_time",
                    items: ["Delivery Time"],
                    keys: ["delivery_time"],
                    sortable: false
                }, {
                    name: "positive_ratio",
                    items: ["Positive Ratio"],
                    keys: ["positive_ratio"],
                    sortable: false
                }, {
                    name: "positive_ratings",
                    items: ["Number of Reviews"],
                    keys: ["positive_ratings"],
                    sortable: false
                }, {
                    name: "country",
                    items: ["Country"],
                    keys: ["country"],
                    sortable: false
                }, {
                    name: "actions",
                    items: ["Action"],
                    keys: ["actions"],
                    sortable: false
                }];

                var columns = [];

                var sortedTd;

                for_(headerData, function(itemGroup) {
                    var name = itemGroup.name;
                    var items = itemGroup.items;
                    var keys = itemGroup.keys;
                    var sortable = itemGroup.sortable;

                    for_(items, function(item, i) {
                        var td = document.createElement("td");

                        td.value = item;
                        td.keyName = keys[i];
                        td.innerHTML = "<div><div>" + item + "</div></div>";
                        header.appendChild(td);
                        var index = columns.length;
                        columns.push(keys[i]);

                        if (sortable) {
                            var sortStatus = 0;
                            td.className = "sortable";

                            dom.on(td, "click", function() {
                                if (sortStatus == 0 || sortedTd != td) {
                                    sortStatus = 1;
                                } else if (sortStatus == -1) {
                                    sortStatus = 0;
                                } else {
                                    sortStatus = -1;
                                }

                                if (sortStatus == 1) {
                                    td.className = "sortable asc-sorted";
                                } else if (sortStatus == -1) {
                                    td.className = "sortable desc-sorted";
                                } else {
                                    td.className = "sortable";
                                }

                                if (sortStatus != 0) {
                                    if (sortedTd != td) {
                                        if (sortedTd) {
                                            sortedTd.className = "sortable";
                                        }
                                        sortedTd = td;
                                    }
                                    favoritesTables[tab_id].sortBy(index, sortStatus == 1);
                                } else {
                                    sortedTd = null;
                                    favoritesTables[tab_id].restoreOrder();
                                }
                            });
                        }
                    });
                });

                trBuilder.setColumns(columns);

                favoritesTables[tab_id].setHeader(header);
                favoritesTables[tab_id].clear();

                $('#favorites-table-outter-' + tab_id + ' table tr td:eq(0)').css('width', '130px');
                $('#favorites-table-outter-' + tab_id + ' table tr td:eq(6)').css('width', '190px');

                //air.trace(tab_id);

                var tab = favoritesMod.getFavoriteByTabId(tab_id);
                //air.trace(JSON.stringify(tab));

                if (tab) {
                    if (tab.elements.length > 0) {
                        for (var i in tab.elements) {
                            var element = tab.elements[i];

                            favoritesMod.addTabElementToTable(tab_id, element);
                        }

                        favoritesTables[tab_id].insert(trsFavoritesMaps[tab_id]);
                        favoritesTables[tab_id].toFirst();

                        favoritesMod.activateActionButtons();
                    }
                }
            };

            this.activateActionButtons = function() {
                $('.favorite-site-delete').each(function(event) {
                    $(this).unbind("click");

                    $(this).click(function(event) {
                        var this_position = $(this).attr('data-position');
                        var tab_id = $(this).attr('tab-id');

                        var favorites = favoritesMod.getFavoritesFromDB();
                        for (var i in favorites) {
                            if (tab_id == favorites[i].id) {
                                for (var j in favorites[i].elements) {
                                    var tab_element = favorites[i].elements[j];
                                    if (j == this_position) {
                                        favorites[i].elements.splice(j, 1);
                                    }
                                }
                            }
                        }

                        var favoritesFile = new air.File("app-storage:/favorites");
                        file.writeText(favoritesFile, JSON.stringify(favorites));

                        var selectedTabIndex = $("#favorites-tabs").tabs('option', 'selected');
                        favoritesTables = [];
                        tabsInnitted = [];

                        $("#favorites-tabs").replaceWith('<article id="favorites-tabs" class="tabs" style="padding-top: 15px;"><ul></ul></article>');
                        favoritesMod.init();

                        $("#favorites-tabs ul li:eq(" + selectedTabIndex + ")>a:eq(0)").click();

                        favoritesMod.activateActionButtons();
                    });
                });

                $('.favorite-site-fiverr').each(function(event) {
                    $(this).unbind("click");

                    $(this).click(function(event) {
                        $("#search-in-fiverr").click();
                        var site_id = $(this).attr('data-site-id');
                        var keyword_category = $(this).attr('data-keyword');
                        var data_type = $(this).attr('data-type');
                        var filters = $(this).attr('data-filters');

                        use_("ui", function(ui) {
                            ui.data("common.currentTab", 0);
                        });

                        $("#fiverr-category").select2("val", "0");
                        $("#fiverr-subcategory").select2("val", "0");

                        if (data_type == "keyword") {
                            $("#fiver-keyword").val(keyword_category);
                        } else {
                            $("#fiver-keyword").val("");
                            var exploded = keyword_category.split("-");
                            if (exploded.length == 2) {
                                $("#fiverr-category").select2("val", $("#fiverr-category option:contains(" + trim(exploded[0]) + ")").val());
                                $('#fiverr-category').trigger('change');
                                $("#fiverr-subcategory").select2("val", $("#fiverr-subcategory option:contains(" + trim(exploded[1]) + ")").val());
                            } else {
                                $("#fiverr-category").select2("val", $("#fiverr-category option:contains(" + keyword_category + ")").val());
                            }
                        }

                        if (!IsEmpty(filters)) {
                            filters = decodeURIComponent(filters);
                            filters = $.parseJSON(filters);

                            $("#filter-delivery-time").select2("val", (IsEmpty(filters.delivery_time.value)) ? "" : filters.delivery_time.value);
                            $("#filter-positive-ratio").select2("val", (IsEmpty(filters.positive_ratio.value)) ? "" : filters.positive_ratio.value);
                            $("#filter-positive-ratings").select2("val", (IsEmpty(filters.positive_ratings.value)) ? "" : filters.positive_ratings.value);
                            $("#filter-country").select2("val", (IsEmpty(filters.country.value)) ? "" : filters.country.value);
                        }

                        $("#search-btn").click();
                    });
                });

                $('.favorite-site-seoclerks').each(function(event) {
                    $(this).unbind("click");

                    $(this).click(function(event) {
                        $("#search-in-seoclerks").click();
                        var site_id = $(this).attr('data-site-id');
                        var keyword_category = $(this).attr('data-keyword');
                        var data_type = $(this).attr('data-type');
                        var filters = $(this).attr('data-filters');

                        use_("ui", function(ui) {
                            ui.data("common.currentTab", 0);
                        });

                        air.trace(data_type);

                        $("#seoclerks-category").select2("val", "0");
                        $("#seoclerks-subcategory").select2("val", "0");

                        if (data_type == "keyword") {
                            $("#seoclerks-keyword").val(keyword_category);
                        } else {
                            $("#seoclerks-keyword").val('');
                            var exploded = keyword_category.split("-");
                            if (exploded.length == 2) {
                                $("#seoclerks-category").select2("val", $("#seoclerks-category option:contains(" + trim(exploded[0]) + ")").val());
                                $('#seoclerks-category').trigger('change');
                                $("#seoclerks-subcategory").select2("val", $("#seoclerks-subcategory option:contains(" + trim(exploded[1]) + ")").val());
                            } else {
                                $("#seoclerks-keyword").val('');
                                $("#seoclerks-category").select2("val", $("#seoclerks-category option:contains(" + keyword_category + ")").val());
                            }
                        }

                        if (!IsEmpty(filters)) {
                            filters = decodeURIComponent(filters);
                            filters = $.parseJSON(filters);

                            $("#filter-delivery-time").select2("val", (IsEmpty(filters.delivery_time.value)) ? "" : filters.delivery_time.value);
                            $("#filter-positive-ratio").select2("val", (IsEmpty(filters.positive_ratio.value)) ? "" : filters.positive_ratio.value);
                            $("#filter-positive-ratings").select2("val", (IsEmpty(filters.positive_ratings.value)) ? "" : filters.positive_ratings.value);
                            $("#filter-country").select2("val", (IsEmpty(filters.country.value)) ? "" : filters.country.value);
                        }

                        $("#search2-btn").click();
                    });
                });

                $('.edit-button').each(function(event) {
                    $(this).unbind("click");

                    $(this).click(function(event) {
                        var site_id = $(this).attr('data-site-id');
                        var keyword_category = $(this).attr('data-keyword');
                        var data_type = $(this).attr('data-type');
                        var filters = $(this).attr('data-filters');

                        $("#favorites-site").select2("val", site_id);
                        $("#favorites-site").trigger('change');
                        $("#favorites-category").select2("val", "0");
                        $("#favorites-subcategory").select2("val", "0");

                        if (data_type == "keyword") {
                            $("#favorites-keyword").val(keyword_category);
                        } else {
                            $("#favorites-keyword").val("");
                            var exploded = keyword_category.split("-");
                            if (exploded.length == 2) {
                                $("#favorites-category").select2("val", $("#favorites-category option:contains(" + trim(exploded[0]) + ")").val());
                                $('#favorites-category').trigger('change');
                                $("#favorites-subcategory").select2("val", $("#favorites-subcategory option:contains(" + trim(exploded[1]) + ")").val());
                            } else {
                                $("#favorites-category").select2("val", $("#favorites-category option:contains(" + keyword_category + ")").val());
                            }
                        }

                        if (!IsEmpty(filters)) {
                            filters = decodeURIComponent(filters);
                            filters = $.parseJSON(filters);

                            $("#favorites-filter-delivery-time").select2("val", (IsEmpty(filters.delivery_time.value)) ? "" : filters.delivery_time.value);
                            $("#favorites-filter-positive-ratio").select2("val", (IsEmpty(filters.positive_ratio.value)) ? "" : filters.positive_ratio.value);
                            $("#favorites-filter-positive-ratings").select2("val", (IsEmpty(filters.positive_ratings.value)) ? "" : filters.positive_ratings.value);
                            $("#favorites-filter-country").select2("val", (IsEmpty(filters.country.value)) ? "" : filters.country.value);
                        }

                        var element_position = parseInt($(this).parents('tr').index());
                        $("#edit-favorite-position").val(element_position);

                        $("#favorites-add-favorite").hide();
                        $("#favorites-edit-save-favorite").show();
                    });
                });
            };

            this.initAddTabButton = function() {
                var favorites = favoritesMod.getFavoritesFromDB();
                if (favorites.length >= 9)
                    return false;

                if ($("#addTab").length == 0) {
                    $("#favorites-tabs>ul").append('<li id="addTab"><a href="javascript: void(0);">+</a></li>');

                    $("#addTab").click(function() {
                        favoritesMod.addNewTab();
                    });
                }
            };

            this.addNewTab = function() {
                $("#addTab").remove();
                $("#favorites-tabs>ul").append('<li id="addTabInput"><input type="text" id="new_tab_name_input" value="New Tab" style="width: 70px;" maxlength="20" /></li>');
                $("#new_tab_name_input").select();

                $("#new_tab_name_input").keypress(function(e) {
                    if (e.keyCode == 13) {
                        //this.blur();
                        favoritesMod.saveTab($(this).val());
                    }
                });

                $("#new_tab_name_input").blur(function(e) {
                    $("#addTabInput").remove();
                    favoritesMod.initAddTabButton();
                });
            };

            this.saveTab = function(tab_name) {
                $("#addTabInput").remove();

                var favorites = favoritesMod.getFavoritesFromDB();
                var tab_id = convertToSlug(tab_name);

                if ($('#tabs-left-' + tab_id).length > 0) {
                    $("#favorites-tabs").tabs("select", '#tabs-left-' + tab_id);
                    favoritesMod.initTable(tab_id);
                } else {
                    var full_tab_name = '<span class="tab_name jqTooltip" title="double click to edit">' + tab_name + '</span>' +
                        '<span class="tab_edit" style="display: none;"><input type="text" class="tab_name_input" value="' + tab_name + '"  style="width: 70px; height: 20px;" maxlength="15" /></span>';

                    $("#favorites-tabs").tabs("add", '#tabs-left-' + tab_id, full_tab_name);
                    $("#favorites-tabs>ul li a").last().attr("id", 'tab-button-' + tab_id);
                    var the_li = $("#favorites-tabs>ul li").last();
                    the_li.attr("tab-id", tab_id);
                    the_li.attr("title", "double click to edit");
                    the_li.tooltip();

                    $("#tab-button-" + tab_id).parent().append('<a href="javascript:void(0)" class="tabs-close" id="tabs-close' + tab_id + '">x</a>');
                    $("#favorites-tabs #tabs-left-" + tab_id).append('<div id="favorites-table-outter-' + tab_id + '" class="table-outter"></div>');

                    $('#tabs-close' + tab_id).click(function() {
                        favoritesMod.closeTab(tab_id);
                    });

                    $('#tab-button-' + tab_id + ' .tab_name').dblclick(function() {
                        $('#tab-button-' + tab_id + ' .tab_name').hide();
                        $('#tab-button-' + tab_id + ' .tab_edit').show();
                        $('#tab-button-' + tab_id + ' .tab_name_input').select();
                    });

                    $('#tab-button-' + tab_id + ' .tab_name_input').blur(function() {
                        favoritesMod.saveTabName(tab_id);
                    });

                    $('#tab-button-' + tab_id + ' .tab_name_input').keypress(function(e) {
                        if (e.keyCode == 13) {
                            favoritesMod.saveTabName(tab_id);
                        }
                    });

                    $('#tab-button-' + tab_id).click(function() {
                        setTimeout(showThisTable, 10);

                        function showThisTable() {
                            favoritesMod.initTable(tab_id);
                        }
                    });

                    $('#tab-button-' + tab_id).click();

                    var favoritesFile = new air.File("app-storage:/favorites");
                    var favorites = favoritesMod.getFavoritesFromDB();

                    favorites.push({
                        id: tab_id,
                        name: tab_name,
                        elements: []
                    });

                    file.writeText(favoritesFile, JSON.stringify(favorites));
                }

                favoritesMod.initAddTabButton();
                if (favorites.length < 2) $(".tabs-close").hide();
                else $(".tabs-close").show();
                favoritesMod.buildSelectsWithTabs();
            };

            this.getFavoritesFromDB = function() {
                var favoritesFile = new air.File("app-storage:/favorites");
                var favoritesTxt = file.readText(favoritesFile);

                var favorites = (favoritesTxt) ? $.parseJSON(favoritesTxt) : [];

                if (favorites.length == 0) {
                    favorites.push({
                        id: "first-tab",
                        name: "First Tab",
                        elements: []
                    });
                    file.writeText(favoritesFile, JSON.stringify(favorites));
                }

                return favorites;
            };

            this.closeTab = function(tab_id) {
                $("#addTab").remove();
                var el = $('#tabs-close' + tab_id);
                var el_li = el.parents("li");

                $('#favorites-tabs').tabs('remove', el_li.index());

                tabsInnitted[tab_id] = false;

                var new_favorites = [];
                var favorites = favoritesMod.getFavoritesFromDB();

                for (var i in favorites) {
                    var favorite = favorites[i];
                    if (favorite.id != tab_id) {
                        new_favorites.push(favorite);
                    }
                }

                if (new_favorites.length < 2) $(".tabs-close").hide();
                else $(".tabs-close").show();

                var favoritesFile = new air.File("app-storage:/favorites");
                file.writeText(favoritesFile, JSON.stringify(new_favorites));
                favoritesMod.initAddTabButton();

                favoritesMod.buildSelectsWithTabs();

                var selectedTabIndex = $("#favorites-tabs").tabs('option', 'selected');
                var selectedTabLi = $("#favorites-tabs ul li:eq(" + selectedTabIndex + ")");
                var tab_id = selectedTabLi.attr("tab-id");

                favoritesMod.initTable(tab_id);
            };

            this.addFavoriteToList = function(site, keyword, data_type, tab_id, filters, favorite_position) {
                if (IsEmpty(site) || site == 0) {
                    message.alert("Please select a site.");
                    return false;
                }

                if (IsEmpty(keyword) || keyword == "Keyword...") {
                    message.alert("Please enter a keyword first.");
                    return false;
                }

                if (!data_type) data_type = "keyword";

                if (!tab_id) {
                    var selectedTabIndex = $("#favorites-tabs").tabs('option', 'selected');
                    var selectedTabLi = $("#favorites-tabs ul li:eq(" + selectedTabIndex + ")");
                    var tab_id = selectedTabLi.attr("tab-id");
                }

                var tab = favoritesMod.getFavoriteByTabId(tab_id);
                var new_element = {
                    site: site
                };

                if (data_type == "category") {
                    new_element.category = keyword;
                } else {
                    new_element.keyword = keyword;
                }

                new_element.filters = filters;

                if (favorite_position) {
                    for (var i in tab.elements) {
                        if (favorite_position == (parseInt(i) + 1)) {
                            tab.elements[i] = new_element;
                        }
                    }
                } else {
                    tab.elements.push(new_element);
                }

                //if (favoritesTables[tab_id]) {
                use_("ui", function(ui) {
                    ui.data("common.currentTab", 2);
                });
                favoritesMod.selectTabById(tab_id);
                favoritesMod.initTable(tab_id);
                favoritesMod.addTabElementToTable(tab_id, new_element);
                //}

                favoritesMod.activateActionButtons();

                var favorites = favoritesMod.getFavoritesFromDB();

                for (var i in favorites) {
                    if (tab_id == favorites[i].id) {
                        favorites[i] = tab;
                    }
                }

                var favoritesFile = new air.File("app-storage:/favorites");
                file.writeText(favoritesFile, JSON.stringify(favorites));

                $("#favorites-site").val("0");
                $("#favorites-keyword").val("Keyword...");

                favoritesMod.refreshTabs();
            };

            this.saveTabName = function(tab_id) {
                var new_tab_name = $('#tab-button-' + tab_id + ' .tab_name_input').val();
                if (!IsEmpty(new_tab_name)) {
                    $('#tab-button-' + tab_id + ' .tab_name').text(new_tab_name);

                    var favorites = favoritesMod.getFavoritesFromDB();
                    for (var i in favorites) {
                        if (tab_id == favorites[i].id) {
                            favorites[i].name = new_tab_name;
                            favorites[i].id = convertToSlug(new_tab_name);
                        }
                    }

                    var favoritesFile = new air.File("app-storage:/favorites");
                    file.writeText(favoritesFile, JSON.stringify(favorites));
                }
                $('#tab-button-' + tab_id + ' .tab_name').show();
                $('#tab-button-' + tab_id + ' .tab_edit').hide();

                favoritesMod.refreshTabs();
            }

            this.refreshTabs = function() {
                var selectedTabIndex = $("#favorites-tabs").tabs('option', 'selected');
                favoritesTables = [];
                tabsInnitted = [];

                $("#favorites-tabs").replaceWith('<article id="favorites-tabs" class="tabs" style="padding-top: 15px;"><ul></ul></article>');
                favoritesMod.init();

                $("#favorites-tabs ul li:eq(" + selectedTabIndex + ")>a:eq(0)").click();

                favoritesMod.activateActionButtons();
            };

            this.addTabElementToTable = function(tab_id, element) {
                element.tab_id = tab_id;
                if (element.keyword) {
                    var exploded = element.keyword.split(" IN ");
                    if (exploded.length > 1) {
                        var keyword_category = element.keyword;
                        data_type = "both";
                    } else {
                        var keyword_category = element.keyword;
                        data_type = "keyword";
                    }
                } else {
                    var keyword_category = element.category;
                    data_type = "category";
                }

                if (element.filters) {
                    filters = (IsEmpty(element.filters.delivery_time.value) ? "N/A" : element.filters.delivery_time.text);
                    filters += " - ";
                    filters += (IsEmpty(element.filters.positive_ratio.value) ? "N/A" : element.filters.positive_ratio.text);
                    filters += " - ";
                    filters += (IsEmpty(element.filters.positive_ratings.value) ? "N/A" : element.filters.positive_ratings.text);
                    filters += " - ";
                    filters += (IsEmpty(element.filters.country.value) ? "N/A" : element.filters.country.text);
                    encoded_filters = encodeURIComponent(JSON.stringify(element.filters));
                } else {
                    filters = "N/A - N/A - N/A - N/A";
                    encoded_filters = "";
                }

                if (element.site == "fiverr") {
                    var class_name = 'favorite-site-fiverr';
                } else if (element.site == "seoclerks") {
                    var class_name = 'favorite-site-seoclerks';
                }

                var site = (element.site == "fiverr") ? "Fiverr" : "Seo Clerks";

                var this_position = trsFavoritesMaps[tab_id].length;

                var trData = {
                    keyword_category: keyword_category,
                    delivery_time: (element.filters) ? (IsEmpty(element.filters.delivery_time.value) ? "N/A" : element.filters.delivery_time.text) : "N/A",
                    positive_ratio: (element.filters) ? (IsEmpty(element.filters.positive_ratio.value) ? "N/A" : element.filters.positive_ratio.text) : "N/A",
                    positive_ratings: (element.filters) ? (IsEmpty(element.filters.positive_ratings.value) ? "N/A" : element.filters.positive_ratings.text) : "N/A",
                    country: (element.filters) ? (IsEmpty(element.filters.country.value) ? "N/A" : element.filters.country.text) : "N/A",
                    site: site,
                    actions: '<button class="' + class_name + ' run-search-button button-small" data-site-id="' + element.site + '" data-keyword="' + keyword_category + '" data-type="' + data_type + '" data-filters="' + encoded_filters + '">Run Search</button> &nbsp; ' +
                        '<button class="edit-button button-small" data-site-id="' + element.site + '" data-keyword="' + keyword_category + '" data-type="' + data_type + '" data-filters="' + encoded_filters + '">Edit</button> &nbsp; ' +
                        '<button class="favorite-site-delete button-small button-red" data-position="' + this_position + '" tab-id="' + tab_id + '">Delete</button>'
                };

                var tr = trBuilder.buildTr();
                tr.setData(trData, tab_id);

                $(tr).find("td:not(:eq(6))").mouseover(function(event) {
                    $("#favorites-preview").show();
                    $("#favorites-preview").html('<table cellpadding="4" cellspacing="0" border="0" class="gig-preview-table">' +
                        '<tr>' +
                        '<td class="left">Keyword/Category:</td>' +
                        '<td class="right">' + keyword_category + '</td>' +
                        '</tr>' +
                        '<tr>' +
                        '<td class="left">Site:</td>' +
                        '<td class="right">' + site + '</td>' +
                        '</tr>' +
                        '<tr>' +
                        '<td class="left">Delivery Time:</td>' +
                        '<td class="right">' + ((element.filters) ? (IsEmpty(element.filters.delivery_time.value) ? "N/A" : element.filters.delivery_time.text) : "N/A") + '</td>' +
                        '</tr>' +
                        '<tr>' +
                        '<td class="left">Positive Ratio:</td>' +
                        '<td class="right">' + ((element.filters) ? (IsEmpty(element.filters.positive_ratio.value) ? "N/A" : element.filters.positive_ratio.text) : "N/A") + '</td>' +
                        '</tr>' +
                        '<tr>' +
                        '<td class="left">Number of Reviews:</td>' +
                        '<td class="right">' + ((element.filters) ? (IsEmpty(element.filters.positive_ratings.value) ? "N/A" : element.filters.positive_ratings.text) : "N/A") + '</td>' +
                        '</tr>' +
                        '<tr>' +
                        '<td class="left">Country:</td>' +
                        '<td class="right">' + ((element.filters) ? (IsEmpty(element.filters.country.value) ? "N/A" : element.filters.country.text) : "N/A") + '</td>' +
                        '</tr>' +
                        '</table>'
                    );

                    $("#favorites-preview").css({
                        top: (event.pageY - 130),
                        left: (event.pageX - 210)
                    });
                });

                $(tr).mouseout(function(event) {
                    $("#favorites-preview").hide();
                    $("#favorites-preview").html('');
                });

                trsFavoritesMaps[tab_id].push(tr);

                //favoritesTables[tab_id].insert(tr);
                //favoritesTables[tab_id].toFirst();
            };

            this.getFavoriteByTabId = function(tab_id) {
                var favorite = null;
                var favorites = favoritesMod.getFavoritesFromDB();

                for (var i in favorites) {
                    var this_favorite = favorites[i];
                    if (tab_id == this_favorite.id) {
                        favorite = this_favorite;
                    }
                }
                return favorite;
            };

            this.buildSelectsWithTabs = function() {
                $("#search-select-tab").html('');

                var favoritesFile = new air.File("app-storage:/favorites");
                var favoritesTxt = file.readText(favoritesFile);

                var favorites = $.parseJSON(favoritesTxt);

                for (var i in favorites) {
                    var favorite = favorites[i];
                    $("#search-select-tab").append('<option value="' + favorite.id + '">' + favorite.name + '</option>');
                }
            };

            this.selectTabById = function(tab_id) {
                $("#favorites-tabs").tabs("select", '#tabs-left-' + tab_id);
            };

            var trBuilder = new function() {
                    var colsMap;
                    var number;

                    this.setColumns = function(columns) {
                        colsMap = {};
                        for_(columns, function(col, i) {
                            colsMap[col] = i;
                        });
                        number = columns.length;
                    };

                    this.buildTr = function() {
                        var tr = document.createElement("tr");
                        for (var i = 0; i < number; i++) {
                            tr.appendChild(document.createElement("td"));
                        }
                        tr.setData = setData;
                        return tr;
                    };

                    function setData(data, tab_id) {
                        var tr = this;
                        var cells = this.cells;

                        var removed = false;

                        forin_(data, function(value, name) {

                            if (colsMap.hasOwnProperty(name)) {
                                var index = colsMap[name];
                                var cell = cells[index];
                                cell.innerHTML = "<div><div>" + value.toString().replace(/\r?\n/g, "<br />") + "</div></div>";
                                cell.value = value;
                                //cell.firstChild.title = value.toString().replace(/<br \/>/g, "\n");
                            }
                        });

                        favoritesTables[tab_id].raiseChange();
                    }
                }();

            var favoriteGigsTable;
            var trsGigsMap = [];

            this.showFavoriteGigs = function() {
                favoriteGigsTable;
                trsGigsMap = [];
                $("#favorite-gigs-table-outter").html("");

                favoriteGigsTable = new enhancedTable.Table(450);
                window.favoriteGigsTable = favoriteGigsTable;
                document.getElementById('favorite-gigs-table-outter').appendChild(favoriteGigsTable.element);
                favoriteGigsTable.initialize();

                jQuery(window).resize(function() {
                    setHeight();
                });

                $("#gigs-favorites-tab").click(function() {
                    setHeight();
                });

                setHeight();

                function setHeight() {
                    var height = document.getElementById("favorites-wrapper").offsetHeight;
                    favoriteGigsTable.setSize(height - 100);
                    $("#favorite-gigs-table-outter table tr td:eq(0)").css('width', '20px');
                    var width = document.getElementById("favorites-wrapper").offsetWidth;
                    var gig_name_width = width / 3;
                    $("#search-results-table-outter table tr td:eq(1)").css('width', gig_name_width + 'px');
                }

                var wrapper = favoriteGigsTable.element;
                var table = wrapper.getElementsByTagName("table")[0];
                var tableWrapper = wrapper.getElementsByClassName("table-wrapper")[0];

                var tooltip = document.createElement("div");
                tooltip.style.position = "fixed";
                tooltip.className = "tooltip";
                dom.hide(tooltip);
                wrapper.appendChild(tooltip);

                dom.on(table, ["mouseover", "click"], function(e) {
                    var target = e.target;

                    while (target && target.tagName != "TD") {
                        target = target.parentNode;
                    }

                    if (target) {
                        var child = target.lastChild;

                        if (child) {
                            var grandChild = child.firstChild;
                            var diff = child.offsetWidth - grandChild.offsetWidth;
                            grandChild.style.right = "auto";

                            if (grandChild.offsetWidth + diff < child.offsetWidth) {
                                dom.hide(tooltip);
                                grandChild.style.right = "";
                                return;
                            }

                            grandChild.style.right = "";
                        } else {
                            dom.hide(tooltip);
                            return;
                        }

                        tooltip.innerHTML = target.innerHTML;
                        dom.show(tooltip);

                        var tdPos = target.getBoundingClientRect();
                        var tablePos = tableWrapper.getBoundingClientRect();

                        var w = tooltip.offsetWidth;
                        var h = tooltip.offsetHeight;

                        if (tdPos.left + w < tablePos.right) {
                            tooltip.style.left = tdPos.left + "px";
                        } else {
                            tooltip.style.left = Math.min(tdPos.right, tablePos.right) - w + "px";
                        }

                        if (tdPos.bottom + h < tablePos.bottom) {
                            tooltip.style.top = tdPos.bottom + "px";
                        } else {
                            tooltip.style.top = tdPos.top - h + "px";
                        }
                    }
                });

                dom.on(table, "mouseout", function(e) {
                    dom.hide(tooltip);
                });

                var header = document.createElement("tr");
                header.className = "table-header";

                var headerData = [{
                    name: "checkbox",
                    items: ['<input type="checkbox" id="favorite-gigs-select-all" />'],
                    keys: ["checkbox"],
                    sortable: false
                }, {
                    name: "description",
                    items: ["Gig/Service"],
                    keys: ["description"],
                    sortable: true
                }, {
                    name: "site",
                    items: ["Site"],
                    keys: ["site"],
                    sortable: true
                }, {
                    name: "name",
                    items: ['User'],
                    keys: ["name"],
                    sortable: true
                }, {
                    name: "rating",
                    items: ["Rating"],
                    keys: ["rating"],
                    sortable: true
                }, {
                    name: "country",
                    items: ["Country"],
                    keys: ["country"],
                    sortable: true
                }, {
                    name: "joined",
                    items: ["Joined"],
                    keys: ["joined"],
                    sortable: true
                }, {
                    name: "delivery",
                    items: ["Delivery (days)"],
                    keys: ["delivery"],
                    sortable: true
                }, {
                    name: "reviews",
                    items: ["Reviews"],
                    keys: ["reviews"],
                    sortable: true
                }];

                var columns = [];

                var sortedTd;

                for_(headerData, function(itemGroup) {
                    var name = itemGroup.name;
                    var items = itemGroup.items;
                    var keys = itemGroup.keys;
                    var sortable = itemGroup.sortable;

                    for_(items, function(item, i) {
                        var td = document.createElement("td");

                        td.value = item;
                        td.keyName = keys[i];
                        td.innerHTML = "<div><div>" + item + "</div></div>";
                        header.appendChild(td);
                        var index = columns.length;
                        columns.push(keys[i]);

                        if (sortable) {
                            var sortStatus = 0;
                            td.className = "sortable";

                            dom.on(td, "click", function() {
                                if (sortStatus == 0 || sortedTd != td) {
                                    sortStatus = 1;
                                } else if (sortStatus == -1) {
                                    sortStatus = 0;
                                } else {
                                    sortStatus = -1;
                                }

                                if (sortStatus == 1) {
                                    td.className = "sortable asc-sorted";
                                } else if (sortStatus == -1) {
                                    td.className = "sortable desc-sorted";
                                } else {
                                    td.className = "sortable";
                                }

                                if (sortStatus != 0) {
                                    if (sortedTd != td) {
                                        if (sortedTd) {
                                            sortedTd.className = "sortable";
                                        }
                                        sortedTd = td;
                                    }
                                    favoriteGigsTable.sortBy(index, sortStatus == 1);
                                } else {
                                    sortedTd = null;
                                    favoriteGigsTable.restoreOrder();
                                }
                            });
                        }
                    });
                });

                trBuilder2.setColumns(columns);

                favoriteGigsTable.setHeader(header);
                favoriteGigsTable.clear();

                $("#favorite-gigs-table-outter table tr td:eq(0)").css('width', '20px');
                var width = document.getElementById("favorites-wrapper").offsetWidth;
                var gig_name_width = width / 3;
                $("#search-results-table-outter table tr td:eq(1)").css('width', gig_name_width + 'px');

                $("input#favorite-gigs-select-all").click(function() {
                    for (var i in trsGigsMap) {
                        var tr = trsGigsMap[i];

                        var jTr = $(tr);

                        if (this.checked) {
                            jTr.find('input[name="favorite-gig-item"]').prop('checked', true);
                        } else {
                            jTr.find('input[name="favorite-gig-item"]').prop('checked', false);
                        }
                    }
                    processCheckboxes();
                });

                var favoriteGigsFile = new air.File("app-storage:/favoriteGigs");
                var favoriteGigsTxt = file.readText(favoriteGigsFile);

                var favoriteGigs = (favoriteGigsTxt) ? $.parseJSON(favoriteGigsTxt) : [];

                for (var i in favoriteGigs) {
                    var result = favoriteGigs[i];

                    var name = (result.name) ? result.name : "";

                    var exploded = name.split('value=\"');
                    var exploded2 = exploded[1].split('\"');
                    var this_url = exploded2[0];

                    var exploded3 = this_url.split('seoclerks');

                    var site = (exploded3.length > 1) ? "Seo Clerks" : "Fiverr";

                    var rating = (result.rating) ? result.rating : 0;
                    var rating_count = (result.reviews) ? result.reviews : 0;
                    var seller_country_name = (result.country) ? result.country : "";

                    var joined = (result.joined) ? result.joined : "";
                    var delivery = ((result.delivery) ? result.delivery : "") + " day(s)";
                    var description = (result.description) ? result.description : "";


                    var trData = {
                        checkbox: '<input type="checkbox" name="favorite-gig-item" value="' + result.gig_id + '" />',
                        site: site,
                        name: name,
                        rating: rating + '%',
                        country: seller_country_name,
                        joined: joined,
                        description: description,
                        delivery: delivery,
                        reviews: rating_count
                    };

                    var tr = trBuilder2.buildTr();
                    trsGigsMap.push(tr);
                    tr.setData(trData);
                }

                favoriteGigsTable.insert(trsGigsMap);
                favoriteGigsTable.toFirst();

                $('#favorite-gigs input[type="checkbox"]').click(function() {
                    processCheckboxes();
                });
            };

            function processCheckboxes() {
                var checkboxes_checked = 0;
                for (var i in trsGigsMap) {
                    var tr = trsGigsMap[i];

                    var jTr = $(tr);

                    if (jTr.find('input[name="favorite-gig-item"]').is(":checked")) {
                        checkboxes_checked++;
                    }
                }

                if (checkboxes_checked > 0) {
                    $("#delete-favorite-gigs-btn").prop('disabled', false);
                    $("#order-favorite-gigs-btn").prop('disabled', false);
                } else {
                    $("#delete-favorite-gigs-btn").prop('disabled', true);
                    $("#order-favorite-gigs-btn").prop('disabled', true);
                }
            }

            var trBuilder2 = new function() {
                    var colsMap;
                    var number;

                    this.setColumns = function(columns) {
                        colsMap = {};
                        for_(columns, function(col, i) {
                            colsMap[col] = i;
                        });
                        number = columns.length;
                    };

                    this.buildTr = function() {
                        var tr = document.createElement("tr");
                        for (var i = 0; i < number; i++) {
                            tr.appendChild(document.createElement("td"));
                        }
                        tr.setData = setData;
                        return tr;
                    };

                    function setData(data) {
                        var tr = this;
                        var cells = this.cells;

                        var removed = false;

                        forin_(data, function(value, name) {

                            if (colsMap.hasOwnProperty(name)) {
                                var index = colsMap[name];
                                var cell = cells[index];
                                cell.innerHTML = "<div><div>" + value.toString().replace(/\r?\n/g, "<br />") + "</div></div>";
                                cell.value = value;
                                //cell.firstChild.title = value.toString().replace(/<br \/>/g, "\n");
                            }
                        });

                        favoriteGigsTable.raiseChange();
                    }
                }();

            this.addSelectedResultsToOrder = function() {
                var results = [];
                var trs = [];
                for (var i in trsGigsMap) {
                    var tr = trsGigsMap[i];
                    var jTr = $(tr);
                    var checkbox = jTr.find('input[name="favorite-gig-item"]');
                    var this_keyword = checkbox.val();

                    if (checkbox.is(':checked') && checkbox.val() != 'undefined' && jTr.is(":visible")) {
                        results.push(this_keyword);
                        trs.push(tr);
                    }
                }

                if (results.length == 0) {
                    message.alert("Please select at least one gig.");
                    return false;
                }

                function addGigs() {
                    use_("ui", function(ui) {
                        ui.data("common.currentTab", 1);
                    });

                    use_("order", function(order) {
                        $("#order-sites").val(0);
                        order.addGigs(trs);
                    });
                }

                setTimeout(addGigs, 100);

                return true;
            };

            //EVENT HANDLERS
            $("#favorites-go-to-fav-searches-btn").click(function() {
                $('#favorite-searches').show();
                $('#favorite-gigs').hide();

                $(this).prop('disabled', true);
                $("#favorites-go-to-fav-gigs-btn").prop('disabled', false);
            });

            $("#favorites-go-to-fav-gigs-btn").click(function() {
                $('#favorite-searches').hide();
                $('#favorite-gigs').show();

                $(this).prop('disabled', true);
                $("#favorites-go-to-fav-searches-btn").prop('disabled', false);

                favoritesMod.showFavoriteGigs();
            });

            $("#add-favorite-gigs-btn").click(function() {
                use_("ui", function(ui) {
                    ui.data("common.currentTab", 0);
                });
            });

            $("#order-favorite-gigs-btn").click(function() {
                favoritesMod.addSelectedResultsToOrder();
            });

            $("#delete-favorite-gigs-btn").click(function() {
                var favoriteGigsFile = new air.File("app-storage:/favoriteGigs");
                var favoriteGigsTxt = file.readText(favoriteGigsFile);

                var all_favoriteGigs = $.parseJSON(favoriteGigsTxt);
                var favoriteGigs = [];

                for (var i in all_favoriteGigs) {
                    var tr = trsGigsMap[i];

                    var jTr = $(tr);

                    if (!jTr.find('input[name="favorite-gig-item"]').is(":checked")) {
                        favoriteGigs.push(all_favoriteGigs[i]);
                    }
                }

                file.writeText(favoriteGigsFile, JSON.stringify(favoriteGigs));

                favoritesMod.showFavoriteGigs();
            });

            $("#favorites-edit-save-favorite").click(function() {
                var fav_keyword = "";
                var tab_id = null;

                var data_type = "keyword";
                var site = $("#favorites-site").val();

                if (IsEmpty(site) || site == "0") {
                    message.alert("Please select a site.");
                    return false;
                }

                var keyword = $("#favorites-keyword").val();
                var category = $("#favorites-category").val();
                var subcategory = $("#favorites-subcategory").val();
                var category_text = $("#favorites-category option:selected").text();
                var subcategory_text = $("#favorites-subcategory option:selected").text();

                if (!IsEmpty(keyword) && keyword != "Keyword...") {
                    fav_keyword = keyword;
                }

                if (category != 0) {
                    fav_keyword = category_text;
                    if (subcategory != 0) {
                        fav_keyword = category_text + " - " + subcategory_text;
                    }
                    var data_type = "category";
                }

                if (IsEmpty(fav_keyword)) {
                    message.alert("Please enter a keyword or select a category.");
                    return false;
                }

                var filters = {
                    delivery_time: {
                        value: $("#favorites-filter-delivery-time").val(),
                        text: $('#favorites-filter-delivery-time option:selected').text()
                    },
                    positive_ratio: {
                        value: $("#favorites-filter-positive-ratio").val(),
                        text: $('#favorites-filter-positive-ratio option:selected').text()
                    },
                    positive_ratings: {
                        value: $("#favorites-filter-positive-ratings").val(),
                        text: $('#favorites-filter-positive-ratings option:selected').text()
                    },
                    country: {
                        value: $("#favorites-filter-country").val(),
                        text: $('#favorites-filter-country option:selected').text()
                    }
                };

                var favorite_position = parseInt($("#edit-favorite-position").val());

                favoritesMod.addFavoriteToList(site, fav_keyword, data_type, tab_id, filters, favorite_position);

                $("#favorites-site").select2("val", "0");
                $("#favorites-keyword").val("Keyword...");
                $("#favorites-category").select2("val", "0");
                $("#favorites-subcategory").select2("val", "0");
                $("#favorites-category").html('<option value="0">Category</option>');
                $("#favorites-subcategory").html('<option value="0">Sub Category</option>');
                $("#favorites-filter-delivery-time").select2("val", "");
                $("#favorites-filter-positive-ratio").select2("val", "");
                $("#favorites-filter-positive-ratings").select2("val", "");
                $("#favorites-filter-country").select2("val", "");
                $("#favorites-filter-country").html('<option value="">Country</option>');

                $("#favorites-add-favorite").show();
                $("#favorites-edit-save-favorite").hide();

                favoritesMod.refreshTabs();
            });
            //EVENT HANDLERS
        });
    });