/// <reference path="/dev/AIRAliases.js" />

require_("script/http-request.js");
require_("script/google-hr.js");
require_("script/file.js");
require_("script/ui.js");
require_("script/vejis/dom.js");
require_("script/vejis/drop.js");
require_("script/vejis/drop.ui.js");
require_("script/table.js");

use_("http-request", "google-hr", "file", "dom", "enhanced-table", "drop", "drop.ui",
function (hr, ghr, file, dom, enhancedTable, drop) {
    module_("order", function () {

        var order = this;
        var gigsTable;
        var gigsMap = {};

        var browserUrls = [];

        var window_opener = null;

        var browser_details = {
            opened: false,
            gig_details: {}
        };

        this.init = function(gigColumnName) {
            gigsMap = {};
            $("#order-gigs-table-outter").html("");
            
            gigsTable = new enhancedTable.Table(450);
            window.gigsTable = gigsTable;
            document.getElementById('order-gigs-table-outter').appendChild(gigsTable.element);
            gigsTable.initialize();

            setHeight();
            
            jQuery(window).resize(function() {
                setHeight();
            });

            $("#order-tab").click(function() {
                setHeight();
            });

            function setHeight() {
                var height = document.getElementById("order-wrapper").offsetHeight;
                gigsTable.setSize((height/6));
                var width = document.getElementById("order-wrapper").offsetWidth;
                var gig_name_width = width/3;
                $("#order-gigs-table-outter table tr td:eq(0)").css('width', gig_name_width + 'px');
                $("#order-gigs-table-outter table tr td:eq(9)").css('width', '70px');
                //$("#order-gigs-table-outter table tr td:eq(0)").css('width', '20px');
            }
            
            var wrapper = gigsTable.element;
            var table = wrapper.getElementsByTagName("table")[0];
            var tableWrapper = wrapper.getElementsByClassName("table-wrapper")[0];
            
            var tooltip = document.createElement("div");
            tooltip.style.position = "fixed";
            tooltip.className = "tooltip";
            dom.hide(tooltip);
            wrapper.appendChild(tooltip);
            
            dom.on(table, ["mouseover", "click"], function (e) {
                var target = e.target;
    
                while (target && target.tagName != "TD") {
                    target = target.parentNode;
                }
    
                if (target) {
                    var child = target.lastChild;
    
                    if (child) {
                        var grandChild = child.firstChild;
                        var diff = child.offsetWidth - grandChild.offsetWidth;
                        grandChild.style.right = "auto";
    
                        if (grandChild.offsetWidth + diff < child.offsetWidth) {
                            dom.hide(tooltip);
                            grandChild.style.right = "";
                            return;
                        }
    
                        grandChild.style.right = "";
                    }
                    else {
                        dom.hide(tooltip);
                        return;
                    }
    
                    tooltip.innerHTML = target.innerHTML;
                    dom.show(tooltip);
    
                    var tdPos = target.getBoundingClientRect();
                    var tablePos = tableWrapper.getBoundingClientRect();
    
                    var w = tooltip.offsetWidth;
                    var h = tooltip.offsetHeight;
    
                    if (tdPos.left + w < tablePos.right) {
                        tooltip.style.left = tdPos.left + "px";
                    }
                    else {
                        tooltip.style.left = Math.min(tdPos.right, tablePos.right) - w + "px";
                    }
    
                    if (tdPos.bottom + h < tablePos.bottom) {
                        tooltip.style.top = tdPos.bottom + "px";
                    }
                    else {
                        tooltip.style.top = tdPos.top - h + "px";
                    }
                }
            });
    
            dom.on(table, "mouseout", function (e) {
                dom.hide(tooltip);
            });
            
            var header = document.createElement("tr");
            header.className = "table-header";



            var headerData = [
                {
                    name: "description",
                    items: [gigColumnName],
                    keys: ["description"],
                    sortable: true
                },
                {
                    name: "name",
                    items: ['User'],
                    keys: ["name"],
                    sortable: true
                },
                {
                    name: "rating",
                    items: ["Rating"],
                    keys: ["rating"],
                    sortable: true
                },
                {
                    name: "country",
                    items: ["Country"],
                    keys: ["country"],
                    sortable: true
                },
                {
                    name: "joined",
                    items: ["Joined"],
                    keys: ["joined"],
                    sortable: true
                },
                {
                    name: "delivery",
                    items: ["Delivery (days)"],
                    keys: ["delivery"],
                    sortable: true
                },
                {
                    name: "reviews",
                    items: ["Reviews"],
                    keys: ["reviews"],
                    sortable: true
                },
                {
                    name: "order",
                    items: ["Order"],
                    keys: ["order"],
                    sortable: true
                },
                {
                    name: "save",
                    items: ["Save"],
                    keys: ["save"],
                    sortable: true
                },
                {
                    name: "remove",
                    items: ["Remove"],
                    keys: ["remove"],
                    sortable: true
                }
            ];

            var columns = [];
            
            var sortedTd;
            
            for_(headerData, function (itemGroup) {
                var name = itemGroup.name;
                var items = itemGroup.items;
                var keys = itemGroup.keys;
                var sortable = itemGroup.sortable;

                for_(items, function (item, i) {
                    var td = document.createElement("td");

                    td.value = item;
                    td.keyName = keys[i];
                    td.innerHTML = "<div><div>" + item + "</div></div>";
                    header.appendChild(td);
                    var index = columns.length;
                    columns.push(keys[i]);

                    if (sortable) {
                        var sortStatus = 0;
                        td.className = "sortable";

                        dom.on(td, "click", function () {
                            if (sortStatus == 0 || sortedTd != td) {
                                sortStatus = 1;
                            }
                            else if (sortStatus == -1) {
                                sortStatus = 0;
                            }
                            else {
                                sortStatus = -1;
                            }

                            if (sortStatus == 1) {
                                td.className = "sortable asc-sorted";
                            }
                            else if (sortStatus == -1) {
                                td.className = "sortable desc-sorted";
                            }
                            else {
                                td.className = "sortable";
                            }

                            if (sortStatus != 0) {
                                if (sortedTd != td) {
                                    if (sortedTd) {
                                        sortedTd.className = "sortable";
                                    }
                                    sortedTd = td;
                                }
                                gigsTable.sortBy(index, sortStatus == 1);
                            }
                            else {
                                sortedTd = null;
                                gigsTable.restoreOrder();
                            }
                        });
                    }
                });
            });

            trBuilder.setColumns(columns);

            gigsTable.setHeader(header);
            gigsTable.clear();


            var width = document.getElementById("order-wrapper").offsetWidth;
            var gig_name_width = width/3;
            $("#order-gigs-table-outter table tr td:eq(0)").css('width', gig_name_width + 'px');
            $("#order-gigs-table-outter table tr td:eq(9)").css('width', '70px');
            
            //$("#order-gigs-table-outter table tr td:eq(0)").css('width', '20px');
        };
        
        this.addGigs = function(trs, isJson) {
            document.getElementById("gigOrderFrame").src = "";

            if (isJson) {
                var exploded = trs[0].gig_url.split("seoclerks");
                var gigColumnName = (exploded.length == 1) ? "Gig" : "Service";
            }
            else {
                var gigColumnName = ($("#search-in-fiverr").is(":checked")) ? "Gig" : "Service";
            }
            order.init(gigColumnName);

            $("#video-stats-top-wrapper").show();
            $("#video-stats-analyzed-wrapper").hide();

            var project_id = $("#order-projects").val();

            if (isJson) {
                var tr = trs[0];

                var saveColumn = '<button class="save-button" style="padding-left: 5px; padding-right: 5px; height: 20px; font-size: 12px; margin-top: 3px; position: relative;">Save</button>';

                var projects = order.getProjectsFromDB();

                if (project_id != 0) {
                    for (var j in projects) {
                        var this_project = projects[j];
                        if (this_project.id == project_id && this_project.gigs) {
                            for (var x in this_project.gigs) {
                                var this_gig = this_project.gigs[x];
                                if (this_gig) {
                                    if (strip_tags(this_gig.user) == strip_tags(tr.user)) {
                                        saveColumn = "Saved";
                                    }
                                }
                            }
                        }
                    }
                }

                var this_tr = trBuilder.buildTr();
                this_tr.setData({
                    description: tr.name,
                    name: tr.user + '<input type="hidden" class="gig_url" value="' + tr.gig_url + '" />',
                    rating: tr.rating,
                    country: tr.country,
                    joined: tr.joined,
                    delivery: tr.delivery,
                    reviews: tr.reviews,
                    order: '<button class="order-button" style="padding-left: 5px; padding-right: 5px; height: 20px; font-size: 12px; margin-top: 3px; position: relative;">Order</button>',
                    save: saveColumn,
                    remove: '<button class="remove-button" data-row-nr="0" style="padding-left: 5px; padding-right: 5px; height: 20px; font-size: 12px; margin-top: 3px; position: relative;">Remove</button>'
                });
                
                gigsMap[i] = this_tr;
            }
            else {            
                for (var i in trs) {
                    var tr = trs[i];
                    var jTr = $(tr);
                    var this_tr = trBuilder.buildTr();

                    var name = jTr.find("td:eq(2)").html();
                    name = name.replace("<div><div>", "");
                    name = name.replace("</div></div>", "");

                    var saveColumn = '<button class="save-button" style="padding-left: 5px; padding-right: 5px; height: 20px; font-size: 12px; margin-top: 3px; position: relative;">Save</button>';

                    var projects = order.getProjectsFromDB();

                    this_tr.setData({
                        description: jTr.find("td:eq(1)").text(),
                        name: name,
                        rating: jTr.find("td:eq(3)").text(),
                        country: jTr.find("td:eq(4)").text(),
                        joined: jTr.find("td:eq(5)").text(),
                        delivery: jTr.find("td:eq(6)").text(),
                        reviews: jTr.find("td:eq(7)").text(),
                        order: '<button class="order-button" style="padding-left: 5px; padding-right: 5px; height: 20px; font-size: 12px; margin-top: 3px; position: relative;">Order</button>',
                        save: saveColumn,
                        remove: '<button class="remove-button" data-row-nr="' + i + '" style="padding-left: 5px; padding-right: 5px; height: 20px; font-size: 12px; margin-top: 3px; position: relative;">Remove</button>'
                    });
                    
                    gigsMap[i] = this_tr;
                }
            }
            
            for (var i in gigsMap) {
                var tr = gigsMap[i];
                
                gigsTable.insert(tr);
                
                var jThis_tr = $(tr);

                jThis_tr.find('input[type="checkbox"]').prop('checked', false);
            }

            for (var i in gigsMap) {
                var tr = gigsMap[i];
                var jTr = $(tr);

                jTr.click(function() {
                    for (var i in gigsMap) {
                        var tr = gigsMap[i];
                        var jTr = $(tr);
                        jTr.css('background', 'none');
                    }

                    var thisJTr = $(this);
                    thisJTr.css('background', '#CCCCCC');

                    var gig_url = thisJTr.find(".gig_url").val();

                    var exploded = gig_url.split("seoclerks");

                    if (exploded.length == 1 && gig_url.substr(0, 4) != "http") {
                        gig_url = "http://fiverr.com" + gig_url;
                    }

                    //document.getElementById("gigOrderFrame").src = 'loading.html';

                    setTimeout(openInIframe, 200);

                    function openInIframe() {
                        document.getElementById("gigOrderFrame").src = gig_url;
                    }
                });

                jTr.find('.order-button').click(function(event) {
                    order.orderGigForProject(this);
                    return false;
                });

                jTr.find('.remove-button').click(function(event) {
                    order.removeRowFromOrdersTable(this);
                    return false;
                });
            }

            gigsTable.toFirst();
        };

        this.removeRowFromOrdersTable = function(el) {
            var row_nr = $(el).attr('data-row-nr');
            gigsMap[row_nr].removed = true;
            gigsTable.raiseChange();
        };

        this.showCompletePurchaseBox = function() {
            $("#order-complete-price").val('');
            $("#purchased-box").show();
            $("#order-complete-price").focus();
        }

        this.getProjectsFromDB = function() {
            var projectsFile = new air.File("app-storage:/projects");
            var projectsTxt = file.readText(projectsFile);

            var projects = (projectsTxt) ? $.parseJSON(projectsTxt) : [];

            return projects;
        };

        this.orderGigForProject = function(el) {
            $("#purchased-box").hide();
            var parent = $(el).parents("tr");
            var gig_url = parent.find(".gig_url").val();
            var gig_name = parent.find("td:eq(0)").text();

            var exploded = gig_url.split("seoclerks");

            if (exploded.length == 1 && gig_url.substr(0, 4) != "http") {
                gig_url = "http://fiverr.com" + gig_url;
            }
            else {
                var gig_id = parent.find(".gig_id").val();
                //gig_url = "https://www.seoclerks.com/order?id=" + gig_id;
            }

            browser_details.opened = true;
            browser_details.gig_details = {
                url: gig_url,
                name: gig_name
            };

            var browserOptions = new air.NativeWindowInitOptions();
            var bounds = new air.Rectangle(20, 20, 984, 590);
            htmlBrowser = air.HTMLLoader.createRootWindow(true, browserOptions, true);
            var urlReq = new air.URLRequest(gig_url);
            htmlBrowser.load(urlReq);
            htmlBrowser.stage.nativeWindow.visible = true;
            htmlBrowser.stage.nativeWindow.activate();
            htmlBrowser.stage.nativeWindow.maximize();

            checkBrowserClosed();

            function checkBrowserClosed() {
                try {
                    if (!htmlBrowser.stage.nativeWindow.visible) {
                        order.showCompletePurchaseBox();
                    }
                    else {
                        setTimeout(checkBrowserClosed, 100);
                    }
                }
                catch (err) {
                    order.showCompletePurchaseBox();
                }
            }
        };

        this.saveGigToProject = function(el) {
            var project_id = $("#order-projects").val();

            if (project_id == 0) {
                message.alert('Please select a project first.');
                return false;
            }

            var this_gig = {};

            var btnParent = $(el).parent();
            var jTr = $(el).parents("tr");
            this_gig.gig_url = jTr.find(".gig_url").val();
            var exploded = this_gig.gig_url.split("seoclerks");
            this_gig.site = "seoclerks";
            if (exploded.length == 1) {
                this_gig.gig_url = "http://fiverr.com" + this_gig.gig_url;
                this_gig.site = "fiverr";
            }
            this_gig.name = jTr.find("td:eq(0)").text();
            this_gig.user = jTr.find("td:eq(1)").text();
            this_gig.rating = jTr.find("td:eq(2)").text();
            this_gig.country = jTr.find("td:eq(3)").text();
            this_gig.joined = jTr.find("td:eq(4)").text();
            this_gig.delivery = jTr.find("td:eq(5)").text();
            this_gig.reviews = jTr.find("td:eq(6)").text();

            btnParent.html("Saved");

            var projects = order.getProjectsFromDB();

            for (var i in projects) {
                var this_project = projects[i];
                if (this_project.id == project_id) {
                    if (this_project.gigs) {
                        this_project.gigs.push(this_gig);
                    }
                    else {
                        this_project.gigs = [this_gig];
                    }
                    projects[i] = this_project;
                }
            }

            //air.trace(JSON.stringify(projects));

            var projectsFile = new air.File("app-storage:/projects");
            file.writeText(projectsFile, JSON.stringify(projects));

            use_("projects", function (projects) {
                projects.refresh();
            });
        };

        var trBuilder = new function () {
            var colsMap;
            var number;

            this.setColumns = function (columns) {
                colsMap = {};
                for_(columns, function (col, i) {
                    colsMap[col] = i;
                });
                number = columns.length;
            };

            this.buildTr = function () {
                var tr = document.createElement("tr");
                for (var i = 0; i < number; i++) {
                    tr.appendChild(document.createElement("td"));
                }
                tr.setData = setData;
                return tr;
            };

            function setData(data) {
                var tr = this;
                var cells = this.cells;

                var removed = false;

                forin_(data, function (value, name) {
                    if (colsMap.hasOwnProperty(name)) {
                        var index = colsMap[name];
                        var cell = cells[index];
                        cell.innerHTML = "<div><div>" + value.toString().replace(/\r?\n/g, "<br />") + "</div></div>";
                        cell.value = value;
                        //cell.firstChild.title = value.toString().replace(/<br \/>/g, "\n");
                    }
                });

                gigsTable.raiseChange();
            }
        }();

        //EVENT HANDLERS
        $("#order-clear-btn").click(function () {
            $('#order-gigs-table-outter').html('<div style="padding: 10px;">Use the Search tab to get some gigs first.</div>');
            document.getElementById("gigOrderFrame").src = "";
        });

        $("#order-complete-done").click(function() {
            var price = $("#order-complete-price").val();
            if (!isNumber(price)) {
                message.alert('Please enter a price.');
                return false;
            }

            var gig = browser_details.gig_details;
            gig.url = gig.url.replace("http://fiverr.com", "");

            for (var i in gigsMap) {
                var tr = gigsMap[i];
                var jTr = $(tr);
                var gig_url = $(tr).find('.gig_url').val();
                gig_url = gig_url.replace("http://fiverr.com", "");
                
                if (gig_url == gig.url) {
                    var is_saved = $(tr).find('td:eq(7)').text();
                    var project_id = $("#order-projects").val();

                    if (project_id == 0) {
                        message.alert('Please select a project first.');
                        return false;
                    }

                    var ordersFile = new air.File("app-storage:/orders");
                    var ordersTxt = file.readText(ordersFile);

                    var orders = (ordersTxt) ? $.parseJSON(ordersTxt) : [];

                    var myDate = new Date();

                    var exploded = gig_url.split("seoclerks");
                    if (exploded.length == 1) {
                        gig_url = "http://fiverr.com" + gig_url;
                    }

                    orders.push({
                        gig_url:  gig_url,
                        project_id: project_id,
                        price: price,
                        date: myDate.toDateString()
                    });

                    file.writeText(ordersFile, JSON.stringify(orders));

                    jTr.find('.save-button').click();

                    message.alert('Thank you for ordering!');

                    use_("projects", function (projects) {
                        projects.refresh();
                    });
                    break;
                }
            }

            $("#purchased-box").hide();
        });

        $("#order-complete-cancel").click(function() {
            $("#purchased-box").hide();
        });

        $("#order-projects").change(function() {
            var project_id = $(this).val();

            if (project_id != 0) {
                var projects = order.getProjectsFromDB();

                for (var i in gigsMap) {
                    var tr = gigsMap[i];
                    var jTr = $(tr);
                    var gig_url = jTr.find('.gig_url').val();
                    if (gig_url.substr(0, 1) == "/") {
                        gig_url = "http://fiverr.com" + gig_url;
                    }

                    var is_saved = false;

                    for (var j in projects) {
                        var this_project = projects[j];
                        if (this_project.id == project_id) {
                            if (this_project.gigs) {
                                for (var x in this_project.gigs) {
                                    var this_gig = this_project.gigs[x];
                                    if (this_gig) {
                                        if (this_gig.gig_url == gig_url) {
                                            is_saved = true;
                                            jTr.find('td:eq(8)>div>div').text('Saved');
                                        }
                                    }
                                }
                            }
                        }
                    }

                    if (!is_saved) {
                        jTr.find('td:eq(8)>div>div').html('<button class="save-button" style="padding-left: 5px; padding-right: 5px; height: 20px; font-size: 12px; margin-top: 3px; position: relative;">Save</button>');
                        jTr.find('.save-button').click(function(event) {
                            order.saveGigToProject(this);
                            return false;
                        });
                    }
                }
            }
        });
        //EVENT HANDLERS
    });
});