/// <reference path="/dev/AIRAliases.js" />

require_("script/http-request.js");
require_("script/google-hr.js");
require_("script/file.js");
require_("script/ui.js");
require_("script/vejis/dom.js");
require_("script/vejis/drop.js");
require_("script/vejis/drop.ui.js");
require_("script/table.js");

use_("http-request", "google-hr", "file", "dom", "enhanced-table", "drop", "drop.ui",
    function(hr, ghr, file, dom, enhancedTable, drop) {
        module_("projects", function() {

            //refresh projects
            //var projectsFile = new air.File("app-storage:/projects");
            //file.writeText(projectsFile, "");

            var startTime = new Date();
            var elapsed_time = "0:00:00";

            var projects = this;
            var projectsMod = this;
            var projectsTable;
            var trsProjectsMap = {};

            this.initialized = false;

            this.init = function(ignore_initialized) {

                $("#projects-table-outter").html("");

                projectsTable = new enhancedTable.Table(450);
                window.projectsTable = projectsTable;
                document.getElementById('projects-table-outter').appendChild(projectsTable.element);
                projectsTable.initialize();

                jQuery(window).resize(function() {
                    setHeight();
                    //$("#projects-table-outter table tr td:eq(0)").css('width', '20px');
                });

                $("#projects-tab").click(function() {
                    setHeight();
                });

                setHeight();

                function setHeight() {
                    var height = document.getElementById("projects-wrapper").offsetHeight;
                    projectsTable.setSize(height - 100);
                    $("#projects-table-outter table tr td:eq(0)").css('width', '20px');
                    $("#projects-table-outter table tr td:eq(6)").css('width', '40px');
                }

                var wrapper = projectsTable.element;
                var table = wrapper.getElementsByTagName("table")[0];
                var tableWrapper = wrapper.getElementsByClassName("table-wrapper")[0];

                var tooltip = document.createElement("div");
                tooltip.style.position = "fixed";
                tooltip.className = "tooltip";
                dom.hide(tooltip);
                wrapper.appendChild(tooltip);

                dom.on(table, ["mouseover", "click"], function(e) {
                    var target = e.target;

                    while (target && target.tagName != "TD") {
                        target = target.parentNode;
                    }

                    if (target) {
                        var child = target.lastChild;

                        if (child) {
                            var grandChild = child.firstChild;
                            var diff = child.offsetWidth - grandChild.offsetWidth;
                            grandChild.style.right = "auto";

                            if (grandChild.offsetWidth + diff < child.offsetWidth) {
                                dom.hide(tooltip);
                                grandChild.style.right = "";
                                return;
                            }

                            grandChild.style.right = "";
                        } else {
                            dom.hide(tooltip);
                            return;
                        }

                        tooltip.innerHTML = target.innerHTML;
                        dom.show(tooltip);

                        var tdPos = target.getBoundingClientRect();
                        var tablePos = tableWrapper.getBoundingClientRect();

                        var w = tooltip.offsetWidth;
                        var h = tooltip.offsetHeight;

                        if (tdPos.left + w < tablePos.right) {
                            tooltip.style.left = tdPos.left + "px";
                        } else {
                            tooltip.style.left = Math.min(tdPos.right, tablePos.right) - w + "px";
                        }

                        if (tdPos.bottom + h < tablePos.bottom) {
                            tooltip.style.top = tdPos.bottom + "px";
                        } else {
                            tooltip.style.top = tdPos.top - h + "px";
                        }
                    }
                });

                dom.on(table, "mouseout", function(e) {
                    dom.hide(tooltip);
                });

                var header = document.createElement("tr");
                header.className = "table-header";

                var headerData = [{
                        name: "checkbox",
                        items: ['<input type="checkbox" id="projects-select-all" />'],
                        keys: ["checkbox"],
                        sortable: false
                    }, {
                        name: "title",
                        items: ["Title"],
                        keys: ["title"],
                        sortable: true
                    }, {
                        name: "url",
                        items: ["URL"],
                        keys: ["url"],
                        sortable: true
                    }, {
                        name: "keywords",
                        items: ["Keywords"],
                        keys: ["keywords"],
                        sortable: true
                    }, {
                        name: "budget",
                        items: ["Budget"],
                        keys: ["budget"],
                        sortable: true
                    }, {
                        name: "total_spent",
                        items: ["Total Spent"],
                        keys: ["total_spent"],
                        sortable: true
                    },
                    /*
                {
                    name: "description",
                    items: ["Description"],
                    keys: ["description"],
                    sortable: true
                },
                {
                    name: "notes",
                    items: ["Notes"],
                    keys: ["notes"],
                    sortable: true
                },*/
                    {
                        name: "actions",
                        items: [""],
                        keys: ["actions"],
                        sortable: true
                    }
                ];

                var columns = [];

                var sortedTd;

                for_(headerData, function(itemGroup) {
                    var name = itemGroup.name;
                    var items = itemGroup.items;
                    var keys = itemGroup.keys;
                    var sortable = itemGroup.sortable;

                    for_(items, function(item, i) {
                        var td = document.createElement("td");

                        td.value = item;
                        td.keyName = keys[i];
                        td.innerHTML = "<div><div>" + item + "</div></div>";
                        header.appendChild(td);
                        var index = columns.length;
                        columns.push(keys[i]);

                        if (sortable) {
                            var sortStatus = 0;
                            td.className = "sortable";

                            dom.on(td, "click", function() {
                                if (sortStatus == 0 || sortedTd != td) {
                                    sortStatus = 1;
                                } else if (sortStatus == -1) {
                                    sortStatus = 0;
                                } else {
                                    sortStatus = -1;
                                }

                                if (sortStatus == 1) {
                                    td.className = "sortable asc-sorted";
                                } else if (sortStatus == -1) {
                                    td.className = "sortable desc-sorted";
                                } else {
                                    td.className = "sortable";
                                }

                                if (sortStatus != 0) {
                                    if (sortedTd != td) {
                                        if (sortedTd) {
                                            sortedTd.className = "sortable";
                                        }
                                        sortedTd = td;
                                    }
                                    projectsTable.sortBy(index, sortStatus == 1);
                                } else {
                                    sortedTd = null;
                                    projectsTable.restoreOrder();
                                }
                            });
                        }
                    });
                });

                trBuilder.setColumns(columns);

                projectsTable.setHeader(header);
                projectsTable.clear();

                $("#projects-table-outter table tr td:eq(0)").css('width', '20px');
                $("#projects-table-outter table tr td:eq(6)").css('width', '40px');

                var projectsFile = new air.File("app-storage:/projects");
                var projectsTxt = file.readText(projectsFile);

                var projects = (projectsTxt) ? $.parseJSON(projectsTxt) : [];

                for (var i in projects) {
                    projectsMod.appendProjectToList(projects[i]);
                }

                $("input#projects-select-all").click(function() {
                    for (var i in trsProjectsMap) {
                        var tr = trsProjectsMap[i];

                        var jTr = $(tr);

                        if (this.checked) {
                            jTr.find('input[name="projects-item"]').prop('checked', true);
                        } else {
                            jTr.find('input[name="projects-item"]').prop('checked', false);
                        }
                    }
                });

                processCheckboxes();
            };

            var trBuilder = new function() {
                    var colsMap;
                    var number;

                    this.setColumns = function(columns) {
                        colsMap = {};
                        for_(columns, function(col, i) {
                            colsMap[col] = i;
                        });
                        number = columns.length;
                    };

                    this.buildTr = function() {
                        var tr = document.createElement("tr");
                        for (var i = 0; i < number; i++) {
                            tr.appendChild(document.createElement("td"));
                        }
                        tr.setData = setData;
                        return tr;
                    };

                    function setData(data) {
                        var tr = this;
                        var cells = this.cells;

                        var removed = false;

                        forin_(data, function(value, name) {
                            if (colsMap.hasOwnProperty(name)) {
                                var index = colsMap[name];
                                var cell = cells[index];
                                cell.innerHTML = "<div><div>" + value.toString().replace(/\r?\n/g, "<br />") + "</div></div>";
                                cell.value = value;
                                //cell.firstChild.title = value.toString().replace(/<br \/>/g, "\n");
                            }
                        });

                        projectsTable.raiseChange();
                    }
                }();

            this.saveProject = function() {
                var continueSave = true;
                $("#add-project-form input#projects-title").each(function() {
                    if (IsEmpty($(this).val()) && continueSave) {
                        continueSave = false;
                        message.alert("Please enter title.");
                    }
                });

                var project_budget = $("#projects-budget").val();
                project_budget = project_budget.replace(/\$/g, '');
                project_budget = project_budget.replace(/,/g, "");

                if (isNaN(project_budget) && continueSave) {
                    continueSave = false;
                    message.alert("Please enter a valid budget.");
                }

                if (continueSave) {
                    var projectsFile = new air.File("app-storage:/projects");
                    var projectsTxt = file.readText(projectsFile);

                    var projects = (projectsTxt) ? $.parseJSON(projectsTxt) : [];

                    var project_id = $("#project-id").val();

                    if (project_id == 0) {
                        var d = new Date();
                        var id = parseInt(d.getTime() / 1000);

                        var project = {
                            id: id,
                            title: $("#projects-title").val(),
                            url: $("#projects-url").val(),
                            keywords: $("#projects-keywords").val(),
                            budget: project_budget,
                            description: $("#projects-description").val(),
                            notes: $("#projects-notes").val()
                        };

                        projects.push(project);

                        projectsMod.appendProjectToList(project);

                        file.writeText(projectsFile, JSON.stringify(projects));

                        $("#add-project-form input, #add-project-form textarea").val('');
                        $("#add-project-form").hide();
                        projectsTable.toLast();
                    } else {
                        for (var i in projects) {
                            var project = projects[i];
                            if (project.id == project_id) {
                                projects[i].title = $("#projects-title").val();
                                projects[i].url = $("#projects-url").val();
                                projects[i].keywords = $("#projects-keywords").val();
                                projects[i].budget = project_budget;
                                projects[i].description = $("#projects-description").val();
                                projects[i].notes = $("#projects-notes").val();
                            }
                        }
                        $("#add-project-form").hide();

                        file.writeText(projectsFile, JSON.stringify(projects));
                        projectsMod.init(true);
                    }

                    processCheckboxes();
                }

                this.buildSelectsWithProjects();
            };

            this.appendProjectToList = function(project, tr) {
                var total_budget_spent = 0;
                if (project.gigs) {
                    for (var j in project.gigs) {
                        var this_gig = project.gigs[j];
                        if (this_gig) {
                            var orders_nr = 0;

                            var ordersFile = new air.File("app-storage:/orders");
                            var ordersTxt = file.readText(ordersFile);

                            var orders = (ordersTxt) ? $.parseJSON(ordersTxt) : [];

                            if (orders.length > 0) {
                                for (var i in orders) {
                                    order = orders[i];
                                    if (order.project_id == project.id && order.gig_url == this_gig.gig_url) {
                                        orders_nr++;
                                        total_budget_spent = parseInt(total_budget_spent) + parseInt(order.price);
                                    }
                                }
                            }
                        }
                    }
                }

                if (IsEmpty(project.budget)) {
                    total_budget_spent = "";
                }

                var trData = {
                    checkbox: '<input type="checkbox" name="projects-item" value="' + project.id + '" />',
                    title: project.title,
                    url: (project.url) ? project.url : "",
                    keywords: (project.keywords) ? project.keywords : "",
                    budget: "$" + (project.budget) ? project.budget : "",
                    total_spent: "$" + total_budget_spent,
                    description: (project.description) ? project.description : "",
                    notes: (project.notes) ? project.notes : "",
                    actions: '<button class="edit-button" style="padding-left: 5px; padding-right: 5px; height: 20px; font-size: 12px; margin-top: 3px; position: relative;">Edit</button>'
                };

                var tr = trBuilder.buildTr();
                trsProjectsMap[project.id] = tr;
                tr.setData(trData, project.id);

                projectsTable.insert(tr);
                projectsTable.toFirst();

                var jTr = $(tr);

                jTr.click(function(e) {
                    if (e.target.tagName != "INPUT" && e.target.tagName != "BUTTON") {
                        var jTr = $(this);
                        var project_id = jTr.find('input[type="checkbox"]').val();
                        projectsMod.showGigs(project_id, jTr);
                    }
                });

                jTr.find('.edit-button').click(function(e) {
                    for (var i in trsProjectsMap) {
                        var tr = trsProjectsMap[i];
                        var jTr = $(tr);
                        jTr.css('background-color', '');
                        jTr.prop('selected', 'not_selected');
                    }

                    var jTr = $(this).parents('tr');
                    var project_id = jTr.find('input[name="projects-item"]').val();

                    $("#add-edit-project-title").text('Edit project');
                    $("#project-id").val(project_id);
                    $("#add-project-form").show();
                    $("#projects-gigs-wrapper").hide();

                    //$("#overlay").show();

                    var projectsFile = new air.File("app-storage:/projects");
                    var projectsTxt = file.readText(projectsFile);

                    var projects = $.parseJSON(projectsTxt);

                    for (var i in projects) {
                        var project = projects[i];
                        if (project.id == project_id) {
                            $("#projects-title").val(project.title);
                            $("#projects-url").val(project.url);
                            $("#projects-keywords").val(project.keywords);
                            $("#projects-budget").val(project.budget);
                            $("#projects-description").val(project.description);
                            $("#projects-notes").val(project.notes);
                        }
                    }
                });
            };

            var gigsTable;
            var gigsTrs = {};
            var gigsTableInitialized = false;

            this.showGigs = function(project_id, mainJtr) {
                $("#add-project-form").hide();
                $("#projects-gigs-wrapper").show();
                $("#projects-details-wrapper").show();
                $("#orders-wrapper").hide();

                if (mainJtr.prop('selected') == "selected") {
                    mainJtr.css('background-color', '');
                    mainJtr.prop('selected', 'not_selected');
                    $("#projects-gigs-wrapper").hide();
                    return false;
                }

                for (var i in trsProjectsMap) {
                    var tr = trsProjectsMap[i];
                    var jTr = $(tr);
                    jTr.css('background-color', '');
                    jTr.prop('selected', 'not_selected');
                }

                mainJtr.css('background-color', '#CCCCCC');
                mainJtr.prop('selected', 'selected');

                $("#overlay").show();

                $("#projects-gigs-table-outter").html("");

                if (!gigsTableInitialized) {
                    //gigsTableInitialized = true;
                    gigsTable = new enhancedTable.Table(450);
                    window.gigsTable = gigsTable;
                    document.getElementById('projects-gigs-table-outter').appendChild(gigsTable.element);
                    gigsTable.initialize();

                    jQuery(window).resize(function() {
                        setHeight();
                        //$("#projects-gigs-table-outter table tr td:eq(0)").css('width', '20px');
                    });

                    $("#projects-tab").click(function() {
                        setHeight();
                    });

                    setHeight();

                    function setHeight() {
                        var height = document.getElementById("projects-wrapper").offsetHeight;
                        var width = document.getElementById("projects-wrapper").offsetWidth;
                        gigsTable.setSize((height / 2 - 68), 456);
                        $("#projects-gigs-table-outter table tr td:eq(1)").css('width', '50px');
                        $("#projects-gigs-table-outter table tr td:eq(2)").css('width', '120px');
                    }

                    var wrapper = gigsTable.element;
                    var table = wrapper.getElementsByTagName("table")[0];
                    var tableWrapper = wrapper.getElementsByClassName("table-wrapper")[0];

                    var tooltip = document.createElement("div");
                    tooltip.style.position = "fixed";
                    tooltip.className = "tooltip";
                    dom.hide(tooltip);
                    wrapper.appendChild(tooltip);

                    dom.on(table, ["mouseover", "click"], function(e) {
                        var target = e.target;

                        while (target && target.tagName != "TD") {
                            target = target.parentNode;
                        }

                        if (target) {
                            var child = target.lastChild;

                            if (child) {
                                var grandChild = child.firstChild;
                                var diff = child.offsetWidth - grandChild.offsetWidth;
                                grandChild.style.right = "auto";

                                if (grandChild.offsetWidth + diff < child.offsetWidth) {
                                    dom.hide(tooltip);
                                    grandChild.style.right = "";
                                    return;
                                }

                                grandChild.style.right = "";
                            } else {
                                dom.hide(tooltip);
                                return;
                            }

                            tooltip.innerHTML = target.innerHTML;
                            dom.show(tooltip);

                            var tdPos = target.getBoundingClientRect();
                            var tablePos = tableWrapper.getBoundingClientRect();

                            var w = tooltip.offsetWidth;
                            var h = tooltip.offsetHeight;

                            if (tdPos.left + w < tablePos.right) {
                                tooltip.style.left = tdPos.left + "px";
                            } else {
                                tooltip.style.left = Math.min(tdPos.right, tablePos.right) - w + "px";
                            }

                            if (tdPos.bottom + h < tablePos.bottom) {
                                tooltip.style.top = tdPos.bottom + "px";
                            } else {
                                tooltip.style.top = tdPos.top - h + "px";
                            }
                        }
                    });

                    dom.on(table, "mouseout", function(e) {
                        dom.hide(tooltip);
                    });

                    var header = document.createElement("tr");
                    header.className = "table-header";

                    var headerData = [{
                        name: "gig",
                        items: ["Gig"],
                        keys: ["gig"],
                        sortable: true
                    }, {
                        name: "orders",
                        items: ["Orders"],
                        keys: ["orders"],
                        sortable: true
                    }, {
                        name: "actions",
                        items: ["Actions"],
                        keys: ["actions"],
                        sortable: false
                    }, ];

                    var columns = [];

                    var sortedTd;

                    for_(headerData, function(itemGroup) {
                        var name = itemGroup.name;
                        var items = itemGroup.items;
                        var keys = itemGroup.keys;
                        var sortable = itemGroup.sortable;

                        for_(items, function(item, i) {
                            var td = document.createElement("td");

                            td.value = item;
                            td.keyName = keys[i];
                            td.innerHTML = "<div><div>" + item + "</div></div>";
                            header.appendChild(td);
                            var index = columns.length;
                            columns.push(keys[i]);

                            if (sortable) {
                                var sortStatus = 0;
                                td.className = "sortable";

                                dom.on(td, "click", function() {
                                    if (sortStatus == 0 || sortedTd != td) {
                                        sortStatus = 1;
                                    } else if (sortStatus == -1) {
                                        sortStatus = 0;
                                    } else {
                                        sortStatus = -1;
                                    }

                                    if (sortStatus == 1) {
                                        td.className = "sortable asc-sorted";
                                    } else if (sortStatus == -1) {
                                        td.className = "sortable desc-sorted";
                                    } else {
                                        td.className = "sortable";
                                    }

                                    if (sortStatus != 0) {
                                        if (sortedTd != td) {
                                            if (sortedTd) {
                                                sortedTd.className = "sortable";
                                            }
                                            sortedTd = td;
                                        }
                                        gigsTable.sortBy(index, sortStatus == 1);
                                    } else {
                                        sortedTd = null;
                                        gigsTable.restoreOrder();
                                    }
                                });
                            }
                        });
                    });

                    trBuilder2.setColumns(columns);

                    gigsTable.setHeader(header);
                    gigsTable.clear();

                    $("#projects-gigs-table-outter table tr td:eq(1)").css('width', '50px');
                    $("#projects-gigs-table-outter table tr td:eq(2)").css('width', '120px');
                }

                gigsTable.clear();

                var projectsFile = new air.File("app-storage:/projects");
                var projectsTxt = file.readText(projectsFile);

                var projects = $.parseJSON(projectsTxt);

                for (var i in projects) {
                    var this_project = projects[i];
                    if (this_project.id == project_id) {
                        var total_budget_spent = 0;
                        if (this_project.gigs) {
                            for (var j in this_project.gigs) {
                                var this_gig = this_project.gigs[j];
                                if (this_gig) {
                                    var tr = trBuilder2.buildTr();
                                    gigsTrs[this_gig.gig_url] = tr;

                                    var orders_nr = 0;

                                    var ordersFile = new air.File("app-storage:/orders");
                                    var ordersTxt = file.readText(ordersFile);

                                    var orders = (ordersTxt) ? $.parseJSON(ordersTxt) : [];

                                    if (orders.length > 0) {
                                        for (var i in orders) {
                                            order = orders[i];
                                            if (order.project_id == projectsMod.getSelectedProjectId() && order.gig_url == this_gig.gig_url) {
                                                orders_nr++;
                                                total_budget_spent = parseInt(total_budget_spent) + parseInt(order.price);
                                            }
                                        }
                                    }

                                    tr.setData({
                                        gig: this_gig.name + '<span class="tr-data" style="display: none;">' + JSON.stringify(this_gig) + '</span>',
                                        orders: orders_nr,
                                        actions: '<button class="order-button button-small">Order</button> &nbsp; ' +
                                            '<button class="remove-button button-small">Remove</button>'
                                    });

                                    gigsTable.insert(tr);
                                    gigsTable.toFirst();
                                }
                            }
                        }

                        if (IsEmpty(this_project.budget)) {
                            this_project.budget = "";
                            total_budget_spent = "";
                            budget_left = "";
                        } else {
                            var budget_left = parseInt(this_project.budget) - parseInt(total_budget_spent);
                        }

                        $("#project-id-wrapper").val(this_project.id);
                        $("#project-title-wrapper").html(this_project.title);
                        $("#project-url-wrapper").html(this_project.url);
                        $("#project-keywords-wrapper").html(this_project.keywords);
                        $("#project-budget-wrapper").html('<strong>$' + this_project.budget + '</strong>&nbsp;&nbsp;&nbsp;|&nbsp;&nbsp;&nbsp;Total spent: <strong>$' + total_budget_spent + '</strong>&nbsp;&nbsp;&nbsp;|&nbsp;&nbsp;&nbsp;Budget left: <strong>$' + budget_left + '</strong>');
                        $("#project-description-wrapper").html(this_project.description);
                        $("#project-notes-wrapper").html((this_project.notes) ? this_project.notes : "");
                    }
                }

                for (var i in gigsTrs) {
                    var tr = gigsTrs[i];
                    var jTr = $(tr);

                    jTr.find('.order-button').click(function(event) {
                        var jTr = $(this).parents('tr');
                        var trData = jTr.find('.tr-data').text();

                        use_("ui", function(ui) {
                            ui.data("common.currentTab", 1);
                        });

                        use_("order", function(order) {
                            var project_id = projectsMod.getSelectedProjectId();

                            $("#order-projects").val(project_id);
                            order.addGigs([$.parseJSON(trData)], true);
                        });

                    });

                    jTr.find('.remove-button').click(function(event) {
                        var jTr = $(this).parents('tr');
                        var trData = $.parseJSON(jTr.find('.tr-data').text());
                        var gig_url = trData.gig_url;

                        for (var i in projects) {
                            var this_project = projects[i];
                            if (this_project.id == project_id) {
                                if (this_project.gigs) {
                                    for (var j in this_project.gigs) {
                                        var this_gig = this_project.gigs[j];

                                        if (this_gig) {
                                            if (this_gig.gig_url == gig_url) {
                                                delete projects[i].gigs[j];
                                                jTr.remove();
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        var projectsFile = new air.File("app-storage:/projects");
                        file.writeText(projectsFile, JSON.stringify(projects));

                        return false;
                    });
                }
            }

            var trBuilder2 = new function() {
                    var colsMap;
                    var number;

                    this.setColumns = function(columns) {
                        colsMap = {};
                        for_(columns, function(col, i) {
                            colsMap[col] = i;
                        });
                        number = columns.length;
                    };

                    this.buildTr = function() {
                        var tr = document.createElement("tr");
                        for (var i = 0; i < number; i++) {
                            tr.appendChild(document.createElement("td"));
                        }
                        tr.setData = setData;
                        return tr;
                    };

                    function setData(data) {
                        var tr = this;
                        var cells = this.cells;

                        var removed = false;

                        forin_(data, function(value, name) {
                            if (colsMap.hasOwnProperty(name)) {
                                var index = colsMap[name];
                                var cell = cells[index];
                                cell.innerHTML = "<div><div>" + value.toString().replace(/\r?\n/g, "<br />") + "</div></div>";
                                cell.value = value;
                                //cell.firstChild.title = value.toString().replace(/<br \/>/g, "\n");
                            }
                        });

                        gigsTable.raiseChange();
                    }
                }();

            var ordersTable;
            var ordersTrs = [];

            this.showOrdersForProject = function(project_id, mainJtr) {
                ordersTrs = [];
                ordersTable = null;

                $("#projects-details-wrapper").hide();
                $("#orders-wrapper").show();
                $("#projects-gigs-orders-table-outter").html('');

                //gigsTableInitialized = true;
                ordersTable = new enhancedTable.Table(450);
                window.ordersTable = ordersTable;
                document.getElementById('projects-gigs-orders-table-outter').appendChild(ordersTable.element);
                ordersTable.initialize();

                jQuery(window).resize(function() {
                    setHeight();
                    //$("#projects-gigs-orders-table-outter table tr td:eq(0)").css('width', '20px');
                });

                $("#projects-tab").click(function() {
                    setHeight();
                });

                setHeight();

                function setHeight() {
                    var height = document.getElementById("projects-wrapper").offsetHeight;
                    var width = document.getElementById("projects-wrapper").offsetWidth;
                    ordersTable.setSize((height / 2 - 100), 434);
                    $("#projects-gigs-orders-table-outter table tr td:eq(2)").css('width', '60px');
                }

                var wrapper = ordersTable.element;
                var table = wrapper.getElementsByTagName("table")[0];
                var tableWrapper = wrapper.getElementsByClassName("table-wrapper")[0];

                var tooltip = document.createElement("div");
                tooltip.style.position = "fixed";
                tooltip.className = "tooltip";
                dom.hide(tooltip);
                wrapper.appendChild(tooltip);

                dom.on(table, ["mouseover", "click"], function(e) {
                    var target = e.target;

                    while (target && target.tagName != "TD") {
                        target = target.parentNode;
                    }

                    if (target) {
                        var child = target.lastChild;

                        if (child) {
                            var grandChild = child.firstChild;
                            var diff = child.offsetWidth - grandChild.offsetWidth;
                            grandChild.style.right = "auto";

                            if (grandChild.offsetWidth + diff < child.offsetWidth) {
                                dom.hide(tooltip);
                                grandChild.style.right = "";
                                return;
                            }

                            grandChild.style.right = "";
                        } else {
                            dom.hide(tooltip);
                            return;
                        }

                        tooltip.innerHTML = target.innerHTML;
                        dom.show(tooltip);

                        var tdPos = target.getBoundingClientRect();
                        var tablePos = tableWrapper.getBoundingClientRect();

                        var w = tooltip.offsetWidth;
                        var h = tooltip.offsetHeight;

                        if (tdPos.left + w < tablePos.right) {
                            tooltip.style.left = tdPos.left + "px";
                        } else {
                            tooltip.style.left = Math.min(tdPos.right, tablePos.right) - w + "px";
                        }

                        if (tdPos.bottom + h < tablePos.bottom) {
                            tooltip.style.top = tdPos.bottom + "px";
                        } else {
                            tooltip.style.top = tdPos.top - h + "px";
                        }
                    }
                });

                dom.on(table, "mouseout", function(e) {
                    dom.hide(tooltip);
                });

                var header = document.createElement("tr");
                header.className = "table-header";

                var headerData = [{
                    name: "gig",
                    items: ["Gig"],
                    keys: ["gig"],
                    sortable: false
                }, {
                    name: "date",
                    items: ["Date"],
                    keys: ["date"],
                    sortable: false
                }, {
                    name: "amount_paid",
                    items: ["Paid"],
                    keys: ["amount_paid"],
                    sortable: false
                }];

                var columns = [];

                var sortedTd;

                for_(headerData, function(itemGroup) {
                    var name = itemGroup.name;
                    var items = itemGroup.items;
                    var keys = itemGroup.keys;
                    var sortable = itemGroup.sortable;

                    for_(items, function(item, i) {
                        var td = document.createElement("td");

                        td.value = item;
                        td.keyName = keys[i];
                        td.innerHTML = "<div><div>" + item + "</div></div>";
                        header.appendChild(td);
                        var index = columns.length;
                        columns.push(keys[i]);

                        if (sortable) {
                            var sortStatus = 0;
                            td.className = "sortable";

                            dom.on(td, "click", function() {
                                if (sortStatus == 0 || sortedTd != td) {
                                    sortStatus = 1;
                                } else if (sortStatus == -1) {
                                    sortStatus = 0;
                                } else {
                                    sortStatus = -1;
                                }

                                if (sortStatus == 1) {
                                    td.className = "sortable asc-sorted";
                                } else if (sortStatus == -1) {
                                    td.className = "sortable desc-sorted";
                                } else {
                                    td.className = "sortable";
                                }

                                if (sortStatus != 0) {
                                    if (sortedTd != td) {
                                        if (sortedTd) {
                                            sortedTd.className = "sortable";
                                        }
                                        sortedTd = td;
                                    }
                                    ordersTable.sortBy(index, sortStatus == 1);
                                } else {
                                    sortedTd = null;
                                    ordersTable.restoreOrder();
                                }
                            });
                        }
                    });
                });

                trBuilder3.setColumns(columns);

                ordersTable.setHeader(header);

                $("#projects-gigs-orders-table-outter table tr td:eq(2)").css('width', '60px');

                ordersTable.clear();

                var project_id = parseInt($("#project-id-wrapper").val());

                var projectsFile = new air.File("app-storage:/projects");
                var projectsTxt = file.readText(projectsFile);

                var projects = $.parseJSON(projectsTxt);

                var ordersFile = new air.File("app-storage:/orders");
                var ordersTxt = file.readText(ordersFile);

                var orders = (ordersTxt) ? $.parseJSON(ordersTxt) : [];

                for (var i in projects) {
                    var this_project = projects[i];
                    if (this_project.id == project_id) {
                        if (this_project.gigs) {
                            for (var j in this_project.gigs) {
                                var this_gig = this_project.gigs[j];
                                if (this_gig) {
                                    if (orders.length > 0) {
                                        for (var i in orders) {
                                            order = orders[i];
                                            if (order.project_id == project_id && order.gig_url == this_gig.gig_url) {
                                                var tr = trBuilder3.buildTr();
                                                ordersTrs.push(tr);

                                                tr.setData({
                                                    gig: this_gig.name,
                                                    date: order.date,
                                                    amount_paid: '<span class="edit-price-btn jqTooltip" title="double click to edit">$' + order.price + '</span>' +
                                                        '<input type="text" class="edit-price-input" value="' + order.price + '" style="width: 40px; height: 20px; padding: 2px; display: none;" />'
                                                });
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                ordersTable.setTrs(ordersTrs, false);
                ordersTable.toFirst();

                $('.jqTooltip').tooltip();

                for (var i in ordersTrs) {
                    var tr = ordersTrs[i];
                    var jTr = $(tr);

                    jTr.find('.edit-price-btn').dblclick(function(event) {
                        $(this).parents('td').find('.edit-price-btn').hide();
                        $(this).parents('td').find('.edit-price-input').show();
                        $(this).parents('td').find('.edit-price-input').select();
                    });

                    jTr.find('.edit-price-input').keypress(function(e) {
                        if (e.keyCode == 13) {
                            var new_price = this.value;
                            if (isNaN(new_price)) {
                                message.alert('Please enter a valid price!');
                                return false;
                            }

                            var ordersFile = new air.File("app-storage:/orders");
                            var ordersTxt = file.readText(ordersFile);

                            var orders = (ordersTxt) ? $.parseJSON(ordersTxt) : [];

                            var element_position = parseInt($(this).parents('tr').index());

                            var project_id = parseInt($("#project-id-wrapper").val());

                            var order_count = 1;
                            for (var i in orders) {
                                if (orders[i].project_id == project_id) {
                                    if (element_position == order_count) {
                                        orders[i].price = new_price;
                                    }
                                    order_count++;
                                }
                            }
                            file.writeText(ordersFile, JSON.stringify(orders));

                            $(this).parents('td').find('.edit-price-btn').show();
                            $(this).parents('td').find('.edit-price-input').hide();
                            $(this).parents('td').find('.edit-price-btn').html('$' + new_price);
                        }
                    });

                    jTr.find('.edit-price-input').blur(function(e) {
                        $(this).parents('td').find('.edit-price-btn').show();
                        $(this).parents('td').find('.edit-price-input').hide();
                    });
                }
            }

            var trBuilder3 = new function() {
                    var colsMap;
                    var number;

                    this.setColumns = function(columns) {
                        colsMap = {};
                        for_(columns, function(col, i) {
                            colsMap[col] = i;
                        });
                        number = columns.length;
                    };

                    this.buildTr = function() {
                        var tr = document.createElement("tr");
                        for (var i = 0; i < number; i++) {
                            tr.appendChild(document.createElement("td"));
                        }
                        tr.setData = setData;
                        return tr;
                    };

                    function setData(data) {
                        var tr = this;
                        var cells = this.cells;

                        var removed = false;

                        forin_(data, function(value, name) {
                            if (colsMap.hasOwnProperty(name)) {
                                var index = colsMap[name];
                                var cell = cells[index];
                                cell.innerHTML = "<div><div>" + value.toString().replace(/\r?\n/g, "<br />") + "</div></div>";
                                cell.value = value;
                                //cell.firstChild.title = value.toString().replace(/<br \/>/g, "\n");
                            }
                        });

                        ordersTable.raiseChange();
                    }
                }();

            this.showProjectDetails = function() {
                var project_id = $("#project-id-wrapper").val();
                var mainJtr;

                for (var i in trsProjectsMap) {
                    var tr = trsProjectsMap[i];
                    var jTr = $(tr);
                    if (jTr.prop('selected') == "selected") {
                        mainJtr = jTr;
                    }
                }

                $("#overlay").click();
                projectsMod.showGigs(project_id, jTr);
            };

            this.closeGigsPopup = function() {
                for (var i in trsProjectsMap) {
                    var tr = trsProjectsMap[i];
                    var jTr = $(tr);
                    jTr.css('background-color', '');
                    jTr.prop('selected', 'not_selected');
                }
                $("#projects-gigs-wrapper").hide();
            };

            this.deleteProjects = function() {
                var projectsFile = new air.File("app-storage:/projects");
                var projectsTxt = file.readText(projectsFile);

                var all_projects = $.parseJSON(projectsTxt);
                var projects = [];

                for (var i in all_projects) {
                    var tr = trsProjectsMap[all_projects[i].id];

                    var jTr = $(tr);

                    if (!jTr.find('input[name="projects-item"]').is(":checked")) {
                        projects.push({
                            id: all_projects[i].id,
                            title: all_projects[i].title,
                            url: all_projects[i].url,
                            keywords: all_projects[i].keywords,
                            budget: all_projects[i].budget,
                            description: all_projects[i].description,
                            notes: all_projects[i].notes
                        });
                    }
                }

                trsProjectsMap = {};
                projectsTable.clear();

                for (var i in projects) {
                    projectsMod.appendProjectToList(projects[i]);
                }

                file.writeText(projectsFile, JSON.stringify(projects));
                $("#projects-delete-projects-btn").prop('disabled', true);

                this.buildSelectsWithProjects();
                $("#projects-gigs-wrapper").hide();

                processCheckboxes();
            };

            this.refresh = function() {
                $("#projects-gigs-wrapper").hide();

                for (var i in trsProjectsMap) {
                    var tr = trsProjectsMap[i];
                    var jTr = $(tr);
                    jTr.css('background-color', '');
                    jTr.prop('selected', 'not_selected');
                }
            }

            this.buildSelectsWithProjects = function() {
                $("#order-projects").html('<option value="0">Select Project</option>');

                var projectsFile = new air.File("app-storage:/projects");
                var projectsTxt = file.readText(projectsFile);

                var projects = $.parseJSON(projectsTxt);

                for (var i in projects) {
                    var project = projects[i];
                    $("#order-projects").append('<option value="' + project.id + '">' + project.title + '</option>');
                }
            };

            this.getSelectedProjectId = function() {
                var project_id = 0;
                for (var i in trsProjectsMap) {
                    var tr = trsProjectsMap[i];
                    var jTr = $(tr);

                    if (jTr.prop('selected') == "selected") {
                        project_id = jTr.find('input[name="projects-item"]').val();
                    }
                }

                return project_id;
            };

            function processCheckboxes() {
                $('#projects-wrapper input[type="checkbox"]').click(function() {
                    var checkboxes_checked = 0;
                    for (var i in trsProjectsMap) {
                        var tr = trsProjectsMap[i];

                        var jTr = $(tr);

                        if (jTr.find('input[name="projects-item"]').is(":checked")) {
                            checkboxes_checked++;
                        }
                    }

                    if (checkboxes_checked > 0) {
                        $("#projects-delete-projects-btn").prop('disabled', false);
                    } else {
                        $("#projects-delete-projects-btn").prop('disabled', true);
                    }
                });
            }

            //EVENT HANDLERS
            $("#projects-add-project-btn").click(function() {
                $("#add-edit-project-title").text('Add New project');
                $("#project-id").val(0);
                $("#projects-title").val('');
                $("#projects-url").val('');
                $("#projects-keywords").val('');
                $("#projects-budget").val('');
                $("#projects-description").val('');
                $("#projects-notes").val('');
                $("#add-project-form").show();
                $("#projects-gigs-wrapper").hide();
                $("#projects-title").focus();
                //$("#overlay").show();

                for (var i in trsProjectsMap) {
                    var tr = trsProjectsMap[i];
                    var jTr = $(tr);
                    jTr.css('background-color', '');
                    jTr.prop('selected', 'not_selected');
                }
            });
            //EVENT HANDLERS
        });
    });