﻿/// <reference path="/dev/vejis.intellisense.js" />
/// <reference path="/dev/AIRAliases.js" />

require_("script/http-request.js");
require_("script/google-hr.js");
require_("script/file.js");
require_("script/ui.js");
require_("script/vejis/dom.js");
require_("script/vejis/drop.js");
require_("script/vejis/drop.ui.js");
require_("script/table.js");

use_("http-request", "google-hr", "file", "dom", "enhanced-table", "drop", "drop.ui",
    function(hr, ghr, file, dom, enhancedTable, drop) {
        module_("search", function() {

            var startTime = new Date();
            var elapsed_time = "0:00:00";

            var searchMod = this;
            var searches = [];
            var last_search = null;
            var searchResultsTable;
            var trsResultsMap = [];
            var all_results = [];
            var visible_results = [];

            var pagination = {
                selected_page: 1,
                per_page: 0,
                nr_of_pages: 0
            };

            this.init = function(type) {
                var d = new Date();
                last_search = parseInt(d.getTime() / 1000);
                searches[last_search] = false;
                all_results = [];
                $('#search-wrapper button, #search-wrapper input, #search-wrapper select').prop('disabled', true);
                $('#search-results-buttons-stop').prop('disabled', false);

                if (type == "fiverr")
                    this.searchFiverr(last_search);
                else
                    this.searchSEOClerks(last_search);

                visible_results = [];

                this.initPagination();

                $("#gigs-preview").html('');
            };

            this.initTable = function() {
                searchResultsTable = new enhancedTable.Table(450);
                window.searchResultsTable = searchResultsTable;
                document.getElementById('search-results-table-outter').appendChild(searchResultsTable.element);
                searchResultsTable.initialize();

                setHeight();

                jQuery(window).resize(function() {
                    setHeight();
                });

                $("#search-tab").click(function() {
                    setHeight();
                });

                function setHeight() {
                    var height = document.getElementById("search-wrapper").offsetHeight;
                    searchResultsTable.setSize(height - 209);
                    var width = document.getElementById("search-wrapper").offsetWidth;
                    var gig_name_width = width / 3;
                    $("#search-results-table-outter table tr td:eq(1)").css('width', gig_name_width + 'px');

                    if ($("#search-wrapper").is(":visible")) {
                        setTimeout(function() {
                            searchMod.paginate();
                            searchMod.selectPage(pagination.selected_page);
                        }, 500);
                    }
                }

                var wrapper = searchResultsTable.element;
                var table = wrapper.getElementsByTagName("table")[0];
                var tableWrapper = wrapper.getElementsByClassName("table-wrapper")[0];

                var tooltip = document.createElement("div");
                tooltip.style.position = "fixed";
                tooltip.className = "tooltip";
                dom.hide(tooltip);
                wrapper.appendChild(tooltip);

                dom.on(table, ["mouseover", "click"], function(e) {
                    var target = e.target;

                    while (target && target.tagName != "TD") {
                        target = target.parentNode;
                    }

                    if (target) {
                        var child = target.lastChild;

                        if (child) {
                            var grandChild = child.firstChild;
                            var diff = child.offsetWidth - grandChild.offsetWidth;
                            grandChild.style.right = "auto";

                            if (grandChild.offsetWidth + diff < child.offsetWidth) {
                                dom.hide(tooltip);
                                grandChild.style.right = "";
                                return;
                            }

                            grandChild.style.right = "";
                        } else {
                            dom.hide(tooltip);
                            return;
                        }

                        tooltip.innerHTML = target.innerHTML;
                        dom.show(tooltip);

                        var tdPos = target.getBoundingClientRect();
                        var tablePos = tableWrapper.getBoundingClientRect();

                        var w = tooltip.offsetWidth;
                        var h = tooltip.offsetHeight;

                        if (tdPos.left + w < tablePos.right) {
                            tooltip.style.left = tdPos.left + "px";
                        } else {
                            tooltip.style.left = Math.min(tdPos.right, tablePos.right) - w + "px";
                        }

                        if (tdPos.bottom + h < tablePos.bottom) {
                            tooltip.style.top = tdPos.bottom + "px";
                        } else {
                            tooltip.style.top = tdPos.top - h + "px";
                        }
                    }
                });

                dom.on(table, "mouseout", function(e) {
                    dom.hide(tooltip);
                });

                var header = document.createElement("tr");
                header.className = "table-header";

                var gigColumnName = ($("#search-in-fiverr").is(":checked")) ? "Gig" : "Service";

                var headerData = [{
                    name: "checkbox",
                    items: ['<input type="checkbox" id="search-results-select-all" />'],
                    keys: ["checkbox"],
                    sortable: false
                }, {
                    name: "description",
                    items: [gigColumnName],
                    keys: ["description"],
                    sortable: true,
                    type: "string"
                }, {
                    name: "name",
                    items: ['User'],
                    keys: ["name"],
                    sortable: true
                }, {
                    name: "rating",
                    items: ["Rating"],
                    keys: ["rating"],
                    sortable: true,
                    type: "number"
                }, {
                    name: "country",
                    items: ["Country"],
                    keys: ["country"],
                    sortable: true
                }, {
                    name: "joined",
                    items: ["Joined"],
                    keys: ["joined"],
                    sortable: true
                }, {
                    name: "delivery",
                    items: ["Delivery (days)"],
                    keys: ["delivery"],
                    sortable: true,
                    type: "number"
                }, {
                    name: "reviews",
                    items: ["Reviews"],
                    keys: ["reviews"],
                    sortable: true
                }];

                var columns = [];

                var sortedTd;

                for_(headerData, function(itemGroup) {
                    var name = itemGroup.name;
                    var items = itemGroup.items;
                    var keys = itemGroup.keys;
                    var sortable = itemGroup.sortable;
                    var type = itemGroup.type;

                    for_(items, function(item, i) {
                        var td = document.createElement("td");

                        td.value = item;
                        td.keyName = keys[i];
                        td.innerHTML = "<div><div>" + item + "</div></div>";
                        header.appendChild(td);
                        var index = columns.length;
                        columns.push(keys[i]);

                        if (sortable) {
                            var sortStatus = 0;
                            td.className = "sortable";

                            dom.on(td, "click", function() {
                                if (sortStatus == 0 || sortedTd != td) {
                                    sortStatus = 1;
                                } else if (sortStatus == -1) {
                                    sortStatus = 0;
                                } else {
                                    sortStatus = -1;
                                }

                                if (sortStatus == 1) {
                                    td.className = "sortable asc-sorted";
                                } else if (sortStatus == -1) {
                                    td.className = "sortable desc-sorted";
                                } else {
                                    td.className = "sortable";
                                }

                                if (sortStatus != 0) {
                                    if (sortedTd != td) {
                                        if (sortedTd) {
                                            sortedTd.className = "sortable";
                                        }
                                        sortedTd = td;
                                    }
                                    trsResultsMap = (type) ? searchResultsTable.sortBy(index, sortStatus == 1, type) : searchResultsTable.sortBy(index, sortStatus == 1);
                                } else {
                                    sortedTd = null;
                                    trsResultsMap = searchResultsTable.restoreOrder();
                                }
                                searchMod.paginate();
                                searchMod.selectPage(pagination.selected_page);
                            });
                        }
                    });
                });

                trBuilder.setColumns(columns);

                searchResultsTable.setHeader(header);
                searchResultsTable.clear();

                $("#search-results-table-outter table tr td:eq(0)").css('width', '20px');
                var width = document.getElementById("search-wrapper").offsetWidth;
                var gig_name_width = width / 3;
                $("#search-results-table-outter table tr td:eq(1)").css('width', gig_name_width + 'px');

                $("input#search-results-select-all").click(function() {
                    var sa_btn = this;
                    $('#search-results-table-outter input[name="search-results-item"]:visible').prop('checked', sa_btn.checked);

                    var tr_counter = 0;
                    selectTr();

                    function selectTr() {
                        var tr = trsResultsMap[tr_counter];
                        if (!tr)
                            return false;

                        $(tr).find('input[name="search-results-item"]').prop('checked', sa_btn.checked);
                        tr_counter++;

                        setTimeout(selectTr, 2);
                    }

                    searchMod.checkSelected();
                });
            };

            var search_page = null;

            this.searchFiverr = function(last_search) {
                var limit = 10;

                var filter_delivery_time = $("#filter-delivery-time").val();
                var filter_positive_ratio = $("#filter-positive-ratio").val();
                var filter_positive_ratings = $("#filter-positive-ratings").val();
                var filter_country = $("#filter-country option:selected").text();

                search_page = 1;

                trsResultsMap = [];
                var result_counter = 0;

                var url = "";

                var keyword = $("#fiver-keyword").val();
                var fiverr_category = $("#fiverr-category").val();
                var fiverr_subcategory = $("#fiverr-subcategory").val();

                if (!IsEmpty(keyword) && keyword != "Search...") {
                    url = "http://fiverr.com/gigs/gigs_as_json?host=search&type=best_match&query_string=" + encodeURIComponent(keyword) + "&limit=" + limit;
                }

                if (fiverr_category != 0) {
                    url = "http://fiverr.com/gigs/endless_page_as_json?host=category&type=endless_auto&category_id=" + fiverr_category + "&limit=" + limit;
                    if (fiverr_subcategory != 0) {
                        url = "http://fiverr.com/gigs/endless_page_as_json?host=subcategory&type=endless_auto&category_id=" + fiverr_category + "&sub_category_id=" + fiverr_subcategory + "&limit=50";
                    }
                }

                if (IsEmpty(url)) {
                    message.alert("Please enter a keyword or select a category first.");
                    $('#search-wrapper button, #search-wrapper input, #search-wrapper select').prop('disabled', false);
                    $('#search-results-buttons-stop').prop('disabled', true);
                    return false;
                }

                //air.trace(url);

                $("#search-results-table-outter").html('');
                $("#search-results-wrapper").show();

                this.initTable();

                var all_results_counter = 0;
                var valid_results_counter = 0;
                fetchResults();

                //air.navigateToURL(new air.URLRequest(url));

                function fetchResults() {
                    if (searches[last_search]) return false;
                    $('#search-results-wrapper .status').html('<img src="style/images/loading.gif" style="height: 14px;" />');

                    var req = new ghr.Request();

                    var proxiesFile = new air.File("app-storage:/proxies");
                    var savedProxiesTxt = file.readText(proxiesFile);
                    var savedProxies = IsEmpty(savedProxiesTxt) ? [] : $.parseJSON(savedProxiesTxt);

                    var fiverr_proxies = [];

                    for (var i in savedProxies) {
                        if (savedProxies[i].fiverr)
                            fiverr_proxies.push(savedProxies[i].proxy);
                    }

                    api_url = "http://162.209.11.93/api/getGigs/?network=fiverr&url=" + encodeURIComponent(url + "&page=" + search_page);

                    if (fiverr_proxies.length > 0) {
                        var proxy = fiverr_proxies[Math.floor((Math.random() * fiverr_proxies.length))];
                        api_url = url + "&page=" + search_page;
                        var exploded = proxy.split(":");
                        req.defaultProxies = [{
                            host: exploded[0],
                            port: exploded[1]
                        }];
                    }

                    //air.navigateToURL(new air.URLRequest(api_url));

                    //air.trace(url + "&page=" + search_page);
                    //air.trace(api_url);

                    //air.navigateToURL(new air.URLRequest(api_url));

                    req.open("get", api_url);
                    req.send(function() {

                        //air.trace(this.status);

                        if (this.status != 200) {
                            fetchResults();
                            return;
                        }

                        if (searches[last_search]) return false;

                        var html = this.responseText;
                        this.dispose();

                        if (IsEmpty(html)) {
                            fetchResults();
                            return false;
                        }

                        var results = $.parseJSON(html);

                        if (results == null) {
                            fetchResults();
                            return false;
                        }

                        var gigs = results.gigs;

                        if (gigs.length == 0) {
                            done();
                        } else {
                            for (var result_counter in gigs) {
                                if (searches[last_search]) return false;

                                var add_this_gig = true;

                                var result = gigs[result_counter];

                                var rating = ((result.rating) ? result.rating : 0) * 10;
                                var rating_count = (result.rating_count) ? result.rating_count : 0;
                                var seller_country_name = (result.seller_country_name) ? result.seller_country_name : "";

                                if (!IsEmpty(filter_delivery_time))
                                    if (result.duration > filter_delivery_time) add_this_gig = false;

                                var this_positive_ratio = rating;
                                if (!IsEmpty(filter_positive_ratio))
                                    if (this_positive_ratio < filter_positive_ratio) add_this_gig = false;

                                var this_positive_ratings = (this_positive_ratio / 100) * rating_count;
                                if (!IsEmpty(filter_positive_ratings))
                                    if (this_positive_ratings < filter_positive_ratings) add_this_gig = false;

                                if (filter_country != "Country")
                                    if (seller_country_name != filter_country) add_this_gig = false;

                                var name = (result.seller_name) ? result.seller_name : "";
                                name = name + '<input type="hidden" class="gig_url" value="' + result.gig_url + '" />';

                                var this_result = {
                                    gig_id: result.gig_id,
                                    name: name,
                                    rating: rating,
                                    country: seller_country_name,
                                    joined: (result.seller_created_at) ? result.seller_created_at : "",
                                    description: (result.title_full) ? result.title_full : "",
                                    delivery: (result.duration) ? result.duration : "",
                                    reviews: rating_count,
                                    this_positive_ratio: this_positive_ratio,
                                    this_positive_ratings: this_positive_ratings
                                };

                                all_results[all_results_counter] = this_result;

                                var tr = trBuilder.buildTr();
                                trsResultsMap[all_results_counter] = tr;

                                /*<div style="position: absolute; border: solid 1px #50a8de; padding: 10px;>"' +
                                                '<img src="http://cdn3.fiverrcdn.com/photos/283403/v2_162/Apocalyptica02.jpg?1304543693"  alt="write SEO articles for you" />' +
                                            '</div>*/

                                var joined = (result.seller_created_at) ? result.seller_created_at : "";
                                var delivery = ((result.duration) ? result.duration : "") + " day(s)";
                                var description = (result.title_full) ? result.title_full : "";

                                tr.setData({
                                    checkbox: '<input type="checkbox" name="search-results-item" value="' + result.gig_id + '" />',
                                    name: name,
                                    rating: rating + '%',
                                    country: seller_country_name,
                                    joined: joined,
                                    description: '<span class="gig_preview_hover" data-counter="' + all_results_counter + '">' + description + '</span>',
                                    delivery: delivery,
                                    reviews: rating_count
                                });

                                $(tr).find('input[name="search-results-item"]').click(function() {
                                    searchMod.checkSelected();
                                });

                                if (!add_this_gig) {
                                    $(tr).hide();
                                } else {
                                    this_result.key = all_results_counter;
                                    visible_results.push(this_result);

                                    searchMod.paginate();

                                    var this_position = valid_results_counter + 1;
                                    var beginning_position = searchMod.getPaginationBeginningPosition();
                                    var ending_position = searchMod.getPaginationEndingPosition();

                                    if (this_position >= beginning_position && this_position <= ending_position) {
                                        //belongs in this page
                                        valid_results_counter++;
                                    } else {
                                        $(tr).hide();
                                    }
                                }

                                $("#gigs-preview").append('<div class="gig-preview-item gig-preview-' + all_results_counter + '" style="display: none;">' +
                                    '<div class="img_holder">' +
                                    result.img_medium +
                                    '</div>' +
                                    '<div style="float: left; padding-left: 8px; padding-top: 2px;">' +
                                    '<table cellpadding="4" cellspacing="0" border="0" class="gig-preview-table">' +
                                    '<tr>' +
                                    '<td class="left">Name:</td>' +
                                    '<td class="right">' + name + '</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="left">Rating:</td>' +
                                    '<td class="right">' + rating + '%</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="left">Positive Ratings:</td>' +
                                    '<td class="right">' + rating_count + '</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="left">Country:</td>' +
                                    '<td class="right">' + seller_country_name + '</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="left">Joined:</td>' +
                                    '<td class="right">' + joined + '</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="left">Delivery:</td>' +
                                    '<td class="right">' + delivery + '</td>' +
                                    '</tr>' +
                                    '</table>' +
                                    '</div>' +
                                    '<div style="clear: both; padding-top: 5px;">' +
                                    '<span class="description">Description:</span> ' + description +
                                    '</div>' +
                                    '<div style="clear: both;"></div>'
                                );

                                $(tr).find(".gig_preview_hover").mouseover(function(event) {
                                    var counter = $(this).attr("data-counter");
                                    $("#gigs-preview").show();
                                    $('.gig-preview-item').hide();
                                    $(".gig-preview-" + counter).show();

                                    var per_page_items = parseInt($(".gig_preview_hover:visible").length);
                                    var indx = parseInt($(this).closest('table').find('.gig_preview_hover').index($(this)));

                                    if (indx < (per_page_items / 2))
                                        $("#gigs-preview").css({
                                            top: (event.pageY - 130),
                                            left: (event.pageX - 210)
                                        });
                                    else
                                        $("#gigs-preview").css({
                                            top: (event.pageY - 130 - $("#gigs-preview").height()),
                                            left: (event.pageX - 210)
                                        });

                                    $('.tooltip').remove();
                                });

                                $(tr).find(".gig_preview_hover").mouseout(function() {
                                    var counter = $(this).attr("data-counter");
                                    $("#gigs-preview").hide();
                                    $(".gig-preview-" + counter).hide();
                                });

                                all_results_counter++;
                            }
                            search_page++;
                            fetchResults();

                            if ((valid_results_counter - limit) < pagination.per_page)
                                searchResultsTable.setTrs(trsResultsMap);

                            //searchResultsTable.insert(theseResultsMap);
                            //air.trace(trsResultsMap.length);
                        }
                    });

                    req = null;
                }
            };

            this.searchSEOClerks = function(last_search) {
                var filter_delivery_time = $("#filter-delivery-time").val();
                var filter_positive_ratio = $("#filter-positive-ratio").val();
                var filter_positive_ratings = $("#filter-positive-ratings").val();
                var filter_country = $("#filter-country").val();

                search_page = 1;

                trsResultsMap = [];
                var result_counter = 0;

                var url = "";

                var keyword = $("#seoclerks-keyword").val();
                var seoclerks_category = $("#seoclerks-category").val();
                var seoclerks_subcategory = $("#seoclerks-subcategory").val();

                if (!IsEmpty(keyword) && keyword != "Search...") {
                    url = "https://www.seoclerks.com/search/" + encodeURIComponent(keyword);
                }

                if (seoclerks_category != 0) {
                    url = seoclerks_category;
                    if (seoclerks_subcategory != 0) {
                        url = seoclerks_subcategory;
                    }
                }

                if (IsEmpty(url)) {
                    message.alert("Please enter a keyword or select a category first.");
                    $('#search-wrapper button, #search-wrapper input, #search-wrapper select').prop('disabled', false);
                    $('#search-results-buttons-stop').prop('disabled', true);
                    return false;
                }

                $("#search-results-table-outter").html('');
                $("#search-results-wrapper").show();

                this.initTable();

                var all_results_counter = 0;
                var valid_results_counter = 0;

                fetchResults();

                function fetchResults() {
                    $('#search-results-wrapper .status').html('<img src="style/images/loading.gif" style="height: 14px;" />');

                    //air.trace(url + "/page/" + search_page);

                    var proxiesFile = new air.File("app-storage:/proxies");
                    var savedProxiesTxt = file.readText(proxiesFile);
                    var savedProxies = IsEmpty(savedProxiesTxt) ? [] : $.parseJSON(savedProxiesTxt);

                    var seoclerks_proxies = [];

                    for (var i in savedProxies) {
                        if (savedProxies[i].seoclerks)
                            seoclerks_proxies.push(savedProxies[i].proxy);
                    }

                    var api_url = "http://162.209.11.93/api/getGigs/?network=seoclerks&url=" + encodeURIComponent(url + "/page/" + search_page);

                    if (seoclerks_proxies.length > 0) {
                        var proxy = seoclerks_proxies[Math.floor((Math.random() * seoclerks_proxies.length))];
                        api_url = api_url + "&proxy=" + proxy;
                    }

                    //air.navigateToURL(new air.URLRequest(api_url));

                    var req = new air.URLRequest(api_url);
                    var loader = new air.URLLoader();

                    loader.load(req);
                    loader.addEventListener("complete", function() {
                        if (searches[last_search]) return false;

                        var html = loader.data;

                        if (IsEmpty(html)) {
                            fetchResults();
                            return false;
                        }

                        var gigs = [];

                        var exploded = html.split('<li style="min-height:');

                        for (var i in exploded) {
                            if (i > 0) {
                                var item = exploded[i];

                                var seller_username = strip_tags(getValueBetweenStrings(item, '<a href="https://www.seoclerks.com/user/', '"'));
                                var positive_ratings = parseInt(trim(strip_tags(getValueBetweenStrings(item, 'icon-thumbs-up norightmargin">', '</span>'))));
                                var negative_ratings = parseInt(trim(strip_tags(getValueBetweenStrings(item, 'icon-thumbs-down norightmargin">', '</span>'))));
                                var seller_country_name = strip_tags(getValueBetweenStrings(item, '<span class="country ', ' '));
                                var seller_created_at = "N/A";
                                var image_content = strip_tags(getValueBetweenStrings(item, '<img class="lazy"', '/>'));
                                var image_src = strip_tags(getValueBetweenStrings(image_content, 'data-original="', '"'));

                                var title = strip_tags(getValueBetweenStrings(item, '<img class="lazy" alt="', '"'));

                                /*var title = strip_tags(getValueBetweenStrings(item, '<h4 class="h4bit">', '</h4>'));
                            if (IsEmpty(title))
                                var title = strip_tags(getValueBetweenStrings(item, '<h4 class="h4bit premhighlight">', '</h4>'));*/

                                var duration = "N/A";
                                var rating_count = positive_ratings + negative_ratings;
                                var gig_url = getValueBetweenStrings(item, '<a href="', '"');

                                var exploded2 = gig_url.split("/");
                                var gig_id = exploded2[(exploded2.length - 2)];

                                var rating = (positive_ratings / rating_count) * 100;
                                rating = (positive_ratings == 0) ? 0 : rating.toFixed(2);


                                var gig = {
                                    gig_id: gig_id,
                                    seller_name: seller_username,
                                    rating: rating,
                                    positive_ratings: positive_ratings,
                                    negative_ratings: negative_ratings,
                                    seller_country_name: seller_country_name,
                                    seller_created_at: seller_created_at,
                                    title: title,
                                    duration: duration,
                                    rating_count: rating_count,
                                    gig_url: gig_url,
                                    img_medium: '<img src="' + image_src + '" />'
                                };

                                gigs.push(gig);
                            }
                        }

                        //air.trace(gigs.length);

                        if (gigs.length == 0) {
                            done();
                        } else {
                            var total_processed = 0;
                            for (var i = 0; i <= 4; i++) {
                                processNextResult(i);
                            }
                            //processNextResult(0);

                            function processNextResult(result_counter) {
                                if (searches[last_search]) return false;

                                //air.trace(result_counter);

                                if (!gigs[result_counter]) return false;

                                var result = gigs[result_counter];

                                var url = "http://162.209.11.93/api/getGigs/?network=seoclerks&url=" + encodeURIComponent(result.gig_url);

                                var req2 = new air.URLRequest(url);
                                var loader2 = new air.URLLoader();

                                loader2.load(req2);
                                loader2.addEventListener("complete", function() {
                                    if (searches[last_search]) return false;
                                    var html = loader2.data;

                                    if (IsEmpty(html)) {
                                        processNextResult(result_counter);
                                        return false;
                                    }

                                    result.duration = parseInt(trim(strip_tags(getValueBetweenStrings(html, 'Expected delivery of', 'days'))));
                                    continueGigProcessing();
                                });

                                loader2.addEventListener("ioError", function() {
                                    processNextResult(result_counter);
                                });

                                function continueGigProcessing() {
                                    var add_this_gig = true;

                                    var positive_ratings = (result.positive_ratings) ? result.positive_ratings : 0;
                                    var negative_ratings = (result.negative_ratings) ? result.negative_ratings : 0;
                                    var rating_count = (result.rating_count) ? result.rating_count : 0;

                                    var rating = (result.positive_ratings) ? result.positive_ratings : 0;
                                    var seller_country_name = (result.seller_country_name) ? result.seller_country_name : "";

                                    if (!IsEmpty(filter_delivery_time))
                                        if (result.duration > filter_delivery_time) add_this_gig = false;

                                    var this_positive_ratio = (rating_count == 0) ? 0 : ((positive_ratings / rating_count) * 100);
                                    if (!IsEmpty(filter_positive_ratio))
                                        if (this_positive_ratio < filter_positive_ratio) add_this_gig = false;

                                    if (!IsEmpty(filter_positive_ratings))
                                        if (positive_ratings < filter_positive_ratings) add_this_gig = false;

                                    if (!IsEmpty(filter_country))
                                        if (seller_country_name != filter_country) add_this_gig = false;

                                    var name = (result.seller_name) ? result.seller_name : "";
                                    name = name + '<input type="hidden" class="gig_url" value="' + result.gig_url + '" /><input type="hidden" class="gig_id" value="' + result.gig_id + '" />';

                                    var this_result = {
                                        gig_id: result.gig_id,
                                        name: name,
                                        rating: result.rating,
                                        country: seller_country_name,
                                        joined: (result.seller_created_at) ? result.seller_created_at : "",
                                        description: (result.title) ? result.title : "",
                                        delivery: (result.duration) ? result.duration : "",
                                        reviews: rating_count,
                                        this_positive_ratio: this_positive_ratio,
                                        this_positive_ratings: positive_ratings
                                    };

                                    all_results[all_results_counter] = this_result;

                                    var d = new Date(result.create_date * 1000);

                                    var joined = (result.seller_created_at) ? result.seller_created_at : "";
                                    var delivery = ((result.duration) ? result.duration : "") + " day(s)";
                                    var description = (result.title) ? result.title : "";

                                    var tr = trBuilder.buildTr();
                                    trsResultsMap[all_results_counter] = tr;
                                    tr.setData({
                                        checkbox: '<input type="checkbox" name="search-results-item" value="' + result.gig_id + '" />',
                                        name: name,
                                        rating: result.rating + '%',
                                        country: seller_country_name,
                                        joined: joined,
                                        description: '<span class="gig_preview_hover" data-counter="' + all_results_counter + '">' + description + '</span>',
                                        delivery: delivery,
                                        reviews: rating_count
                                    });

                                    $(tr).find('input[name="search-results-item"]').click(function() {
                                        searchMod.checkSelected();
                                    });

                                    searchResultsTable.insert(tr);
                                    searchResultsTable.toFirst();

                                    if (!add_this_gig) {
                                        $(tr).hide();
                                    } else {
                                        this_result.key = all_results_counter;
                                        visible_results.push(this_result);

                                        searchMod.paginate();

                                        var this_position = valid_results_counter + 1;
                                        var beginning_position = searchMod.getPaginationBeginningPosition();
                                        var ending_position = searchMod.getPaginationEndingPosition();

                                        if (this_position >= beginning_position && this_position <= ending_position) {
                                            //belongs in this page
                                            valid_results_counter++;
                                        } else {
                                            $(tr).hide();
                                        }
                                    }



                                    $("#gigs-preview").append('<div class="gig-preview-item gig-preview-' + all_results_counter + '" style="display: none;">' +
                                        '<div class="img_holder">' +
                                        result.img_medium +
                                        '</div>' +
                                        '<div style="float: left; padding-left: 8px; padding-top: 2px;">' +
                                        '<table cellpadding="4" cellspacing="0" border="0" class="gig-preview-table">' +
                                        '<tr>' +
                                        '<td class="left">Name:</td>' +
                                        '<td class="right">' + name + '</td>' +
                                        '</tr>' +
                                        '<tr>' +
                                        '<td class="left">Rating:</td>' +
                                        '<td class="right">' + rating + '%</td>' +
                                        '</tr>' +
                                        '<tr>' +
                                        '<td class="left">Positive Ratings:</td>' +
                                        '<td class="right">' + rating_count + '</td>' +
                                        '</tr>' +
                                        '<tr>' +
                                        '<td class="left">Country:</td>' +
                                        '<td class="right">' + seller_country_name + '</td>' +
                                        '</tr>' +
                                        '<tr>' +
                                        '<td class="left">Joined:</td>' +
                                        '<td class="right">' + joined + '</td>' +
                                        '</tr>' +
                                        '<tr>' +
                                        '<td class="left">Delivery:</td>' +
                                        '<td class="right">' + delivery + '</td>' +
                                        '</tr>' +
                                        '</table>' +
                                        '</div>' +
                                        '<div style="clear: both; padding-top: 5px;">' +
                                        '<span class="description">Description:</span> ' + description +
                                        '</div>' +
                                        '<div style="clear: both;"></div>'
                                    );

                                    $(tr).find(".gig_preview_hover").mouseover(function(event) {
                                        var counter = $(this).attr("data-counter");
                                        $("#gigs-preview").show();
                                        $('.gig-preview-item').hide();
                                        $(".gig-preview-" + counter).show();

                                        var per_page_items = parseInt($(".gig_preview_hover:visible").length);
                                        var indx = parseInt($(this).closest('table').find('.gig_preview_hover').index($(this)));

                                        if (indx < (per_page_items / 2))
                                            $("#gigs-preview").css({
                                                top: (event.pageY - 130),
                                                left: (event.pageX - 210)
                                            });
                                        else
                                            $("#gigs-preview").css({
                                                top: (event.pageY - 130 - $("#gigs-preview").height()),
                                                left: (event.pageX - 210)
                                            });

                                        $('.tooltip').remove();
                                    });

                                    $(tr).find(".gig_preview_hover").mouseout(function() {
                                        var counter = $(this).attr("data-counter");
                                        $("#gigs-preview").hide();
                                        $(".gig-preview-" + counter).hide();
                                    });

                                    total_processed++;

                                    //air.trace(total_processed);

                                    if (total_processed == gigs.length) {
                                        search_page++;
                                        all_results_counter++;
                                        fetchResults();
                                    } else {
                                        all_results_counter++;
                                        processNextResult(result_counter + 5);
                                        //setTimeout(processNextResult, 100);
                                    }
                                }
                            }
                        }
                    });

                    loader.addEventListener("httpStatus", function(e) {
                        //air.trace('httpStatus');
                        //air.trace(e);
                    });

                    loader.addEventListener("ioError", function(e) {
                        //air.trace('error');
                        //air.trace(e);
                        //done();
                        fetchResults();
                        return false;
                    });
                }
            };

            function done() {
                $('#search-results-wrapper .status').html('Done.');
                $('#search-wrapper button, #search-wrapper input, #search-wrapper select').prop('disabled', false);
                $('#search-results-buttons-stop').prop('disabled', true);
                //$("#search-btn").click();
                searchMod.sendResultsToApi(false);
            }

            this.applyFilters = function(search_type) {
                var filter_delivery_time = $("#filter-delivery-time").val();
                var filter_positive_ratio = $("#filter-positive-ratio").val();
                var filter_positive_ratings = $("#filter-positive-ratings").val();
                var filter_country = (search_type == "fiverr") ? $("#filter-country option:selected").text() : $("#filter-country").val();

                visible_results = [];

                for (var i in trsResultsMap) {
                    var tr = trsResultsMap[i];
                    var jTr = $(tr);
                    var result = all_results[i];

                    var add_this_gig = true;

                    if (!IsEmpty(filter_delivery_time))
                        if (result.delivery > filter_delivery_time) add_this_gig = false;

                    if (!IsEmpty(filter_positive_ratio))
                        if (result.this_positive_ratio < filter_positive_ratio) add_this_gig = false;

                    if (!IsEmpty(filter_positive_ratings))
                        if (result.this_positive_ratings < filter_positive_ratings) add_this_gig = false;

                    if (!IsEmpty(filter_country) && filter_country != "Country")
                        if (result.country != filter_country) add_this_gig = false;

                    if (!add_this_gig) {
                        jTr.hide();
                    } else {
                        result.key = i;
                        visible_results.push(result);

                    }
                }

                searchMod.paginate();
                searchMod.selectPage(1);
            };

            this.checkSelected = function() {
                var length = $('#search-results-table-outter input[name="search-results-item"]:checked').length;
                if (length == 0) {
                    $('#search-results-buttons-export').text('Export');
                } else {
                    $('#search-results-buttons-export').text('Export Selected');
                }
            };

            this.addSelectedResultsToOrder = function() {
                var results = [];
                var trs = [];
                for (var i in trsResultsMap) {
                    var tr = trsResultsMap[i];
                    var jTr = $(tr);
                    var checkbox = jTr.find('input[name="search-results-item"]');
                    var this_keyword = checkbox.val();

                    if (checkbox.is(':checked') && checkbox.val() != 'undefined' && jTr.is(":visible")) {
                        results.push(this_keyword);
                        trs.push(tr);
                    }
                }

                if (results.length == 0) {
                    message.alert("Please select at least one gig.");
                    return false;
                }

                function addGigs() {
                    use_("order", function(order) {
                        $("#order-sites").val(0);
                        order.addGigs(trs);
                    });
                }

                setTimeout(addGigs, 100);

                return true;
            };

            this.saveSelectedResultsToFavorites = function() {
                if ($('input[name="search-results-item"]:checked:visible').length == 0) {
                    message.alert("Please select at least one gig.");
                    return false;
                }

                var favorite_results = [];

                for (var i in all_results) {
                    var this_result = all_results[i];

                    if ($('input[value="' + this_result.gig_id + '"]').is(":checked")) {
                        favorite_results.push(this_result);
                    }
                }

                var favoriteGigsFile = new air.File("app-storage:/favoriteGigs");
                var favoriteGigsTxt = file.readText(favoriteGigsFile);

                var favoriteGigs = (favoriteGigsTxt) ? $.parseJSON(favoriteGigsTxt) : [];

                for (var i in favorite_results) {
                    var favorite_result = favorite_results[i];
                    var already_added = false;
                    for (var j in favoriteGigs) {
                        var favorite_gig = favoriteGigs[j];

                        if (favorite_result.gig_id == favorite_gig.gig_id)
                            already_added = true;
                    }

                    if (!already_added) {
                        favoriteGigs.push(favorite_result);
                    }
                }

                file.writeText(favoriteGigsFile, JSON.stringify(favoriteGigs));

                message.alert('Gigs saved as favorites!');

                return true;
            };

            this.initPagination = function() {
                pagination = {
                    selected_page: 1,
                    per_page: 0
                };

                var html = '<div style="float: left; padding-top: 3px;">' +
                    'Showing <span class="search-pagination-showing-start">0</span> to <span class="search-pagination-showing-end">0</span> of <span class="search-pagination-total">0</span> entries' +
                    '</div>' +
                    '<div style="float: right;">' +
                    '<a class="search-pagination-first disabled" href="javascript: void(0);">First</a>' +
                    '<a class="search-pagination-previous disabled" href="javascript: void(0);">Previous</a>';

                html += '<span id="search-pagination-page-numbers"><a class="search-pagination-page1 disabled" href="javascript: void(0);">1</a></span>';

                html += '<a class="search-pagination-next disabled" href="javascript: void(0);">Next</a>' +
                    '<a class="search-pagination-last disabled" href="javascript: void(0);">Last</a>' +
                    '</div>';

                $("#search-results-pagination").html(html);

                $('.search-pagination-first').click(function() {
                    if (!$(this).hasClass("disabled")) searchMod.selectPage(1);
                });

                $('.search-pagination-previous').click(function() {
                    var page_nr = parseInt(pagination.selected_page) - 1;
                    if (!$(this).hasClass("disabled")) searchMod.selectPage(page_nr);
                });

                $('.search-pagination-next').click(function() {
                    var page_nr = parseInt(pagination.selected_page) + 1;
                    if (!$(this).hasClass("disabled")) searchMod.selectPage(page_nr);
                });

                $('.search-pagination-last').click(function() {
                    var page_nr = parseInt(visible_results.length / pagination.per_page) + 1;
                    if (!$(this).hasClass("disabled")) searchMod.selectPage(page_nr);
                });
            };

            this.paginate = function() {
                var table_height = $("#search-results-table-outter").height();
                pagination.per_page = (table_height / 30) - 1;
                pagination.per_page = parseInt(pagination.per_page.toFixed(0));

                $(".search-pagination-total").text(visible_results.length);

                $(".search-pagination-showing-start").text(searchMod.getPaginationBeginningPosition());

                var showing_end = (searchMod.getPaginationEndingPosition() > visible_results.length) ? visible_results.length : searchMod.getPaginationEndingPosition();
                $(".search-pagination-showing-end").text(showing_end);

                var number_of_pages = Math.ceil(visible_results.length / pagination.per_page);

                pagination.nr_of_pages = number_of_pages;

                var pages_html = '';

                var selected_page = parseInt(pagination.selected_page);

                var max = ((selected_page + 2) <= number_of_pages) ? (selected_page + 2) : number_of_pages;
                var min = ((selected_page - 3) > 0) ? (selected_page - 3) : 0;

                if (pagination.selected_page < 3) {
                    max = (number_of_pages < 5) ? number_of_pages : 5;
                }

                if (pagination.selected_page >= (number_of_pages - 1)) {
                    min = (number_of_pages - 5 < 0) ? 0 : (number_of_pages - 5);
                }

                if (max == 0) max = 1;

                for (var i = min + 1; i <= max; i++) {
                    var disabled = (i == pagination.selected_page) ? " disabled" : "";
                    pages_html += '<a class="search-pagination-page search-pagination-page' + i + disabled + '" href="javascript: void(0);">' + i + '</a>';
                }

                $('#search-pagination-page-numbers').html(pages_html);

                $('.search-pagination-page').click(function() {
                    var page_nr = $(this).text();
                    searchMod.selectPage(page_nr);
                });

                if (pagination.selected_page == 1)
                    $('.search-pagination-first').addClass('disabled');
                else
                    $('.search-pagination-first').removeClass('disabled');

                if (pagination.selected_page == 1)
                    $('.search-pagination-previous').addClass('disabled');
                else
                    $('.search-pagination-previous').removeClass('disabled');

                var next_page_nr = (parseInt(pagination.selected_page) + 1);

                if ($('.search-pagination-page' + next_page_nr).length == 0)
                    $('.search-pagination-next').addClass('disabled');
                else
                    $('.search-pagination-next').removeClass('disabled');

                if (pagination.selected_page == number_of_pages || number_of_pages == 0)
                    $('.search-pagination-last').addClass('disabled');
                else
                    $('.search-pagination-last').removeClass('disabled');
            };

            this.selectPage = function(page_nr) {
                var number_of_pages = parseInt(visible_results.length / pagination.per_page) + 1;

                if (page_nr <= 0) {
                    searchMod.selectPage(1);
                    return false;
                }

                if (page_nr > number_of_pages) {
                    searchMod.selectPage(number_of_pages);
                    return false;
                }

                pagination.selected_page = page_nr;
                searchMod.paginate();

                var this_position = 1;
                for (var i in visible_results) {
                    var tr = trsResultsMap[visible_results[i].key];

                    var beginning_position = searchMod.getPaginationBeginningPosition();
                    var ending_position = searchMod.getPaginationEndingPosition();

                    if (this_position >= beginning_position && this_position <= ending_position) {
                        //belongs in this page
                        $(tr).show();
                    } else {
                        $(tr).hide();
                    }
                    this_position++;
                }

                if (searchResultsTable)
                    searchResultsTable.toFirst();
            };

            this.getPaginationBeginningPosition = function() {
                return (visible_results.length == 0) ? 0 : ((pagination.selected_page - 1) * pagination.per_page) + 1;
            };

            this.getPaginationEndingPosition = function() {
                return (pagination.selected_page * pagination.per_page);
            };

            var trBuilder = new function() {
                    var colsMap;
                    var number;

                    this.setColumns = function(columns) {
                        colsMap = {};
                        for_(columns, function(col, i) {
                            colsMap[col] = i;
                        });
                        number = columns.length;
                    };

                    this.buildTr = function() {
                        var tr = document.createElement("tr");
                        for (var i = 0; i < number; i++) {
                            tr.appendChild(document.createElement("td"));
                        }
                        tr.setData = setData;
                        return tr;
                    };

                    function setData(data, raiseChange) {
                        var tr = this;
                        var cells = this.cells;

                        var removed = false;

                        forin_(data, function(value, name) {
                            if (colsMap.hasOwnProperty(name)) {
                                var index = colsMap[name];
                                var cell = cells[index];
                                cell.innerHTML = "<div><div>" + value.toString().replace(/\r?\n/g, "<br />") + "</div></div>";
                                cell.value = value;
                                //cell.firstChild.title = value.toString().replace(/<br \/>/g, "\n");
                            }
                        });

                        if (raiseChange) searchResultsTable.raiseChange();
                    }
                }();

            this.sendResultsToApi = function(manually_stopped) {
                var userFile = new air.File("app-storage:/user");
                var user = file.readText(userFile);

                var url = "http://drew.analyticcalltracking.com/gig_backend/api/saveSearchResults.php";

                var req2 = new air.URLRequest(url);

                var data = new air.URLVariables();
                data["user"] = user;
                data["site"] = ($("#search-in-fiverr").is(":checked")) ? "Fiverr" : "SEO Clerks";

                if (data["site"] == "Fiverr") {
                    var keyword = $("#fiver-keyword").val();
                    var category = $("#fiverr-category option:selected").text();
                    var subcategory = $("#fiverr-subcategory option:selected").text();
                } else {
                    var keyword = $("#seoclerks-keyword").val();
                    var category = $("#seoclerks-category option:selected").text();
                    var subcategory = $("#seoclerks-subcategory option:selected").text();
                }

                if (!IsEmpty(keyword) && keyword != "Search...") {
                    data["keyword_category"] = keyword;
                }
                if (category != 0) {
                    data["keyword_category"] = category;
                    if (subcategory != 0) {
                        data["keyword_category"] = category + " - " + subcategory;
                    }
                }

                data["filters"] = JSON.stringify({
                    filter_delivery_time: $("#filter-delivery-time").val(),
                    filter_positive_ratio: $("#filter-positive-ratio").val(),
                    filter_positive_ratings: $("#filter-positive-ratings").val(),
                    filter_country: $("#filter-country option:selected").text()
                });

                data["search_results"] = JSON.stringify(visible_results);
                data["stopped"] = manually_stopped;


                req2.method = "POST";
                req2.data = data;

                var loader2 = new air.URLLoader();

                loader2.load(req2);
                loader2.addEventListener("complete", function() {
                    air.trace(loader2.data);
                });

                loader2.addEventListener("ioError", function(e) {
                    air.trace(e);
                });
            };

            //EVENT HANDLERS
            $("#search-results-buttons-stop").click(function() {
                searches[last_search] = true;
                $('#search-results-wrapper .status').html('Done.');
                $('#search-wrapper button, #search-wrapper input, #search-wrapper select').prop('disabled', false);
                $('#search-results-buttons-stop').prop('disabled', true);
                searchMod.sendResultsToApi(true);
            });

            $("#search-results-buttons-clear").click(function() {
                $('#search-results-wrapper').hide();
            });

            $('#search-results-buttons-export').click(function() {
                var totalVideos = 0;
                for (var i in trsResultsMap) {
                    totalVideos++;
                }

                if (totalVideos == 0) {
                    message.alert("Nothing to export.");
                    return;
                }

                var length = $('#search-results-table-outter input[name="search-results-item"]:checked').length;

                var csvText = searchResultsTable.toCSV(length);

                var docStorage = air.File.documentsDirectory;

                var myDate = new Date();

                var target = file.getAvailableFile(docStorage, "GIG-search-results-" + (myDate.getMonth() + 1) + "-" + myDate.getDate() + "-" + myDate.getFullYear() + ".csv");

                target.browseForSave("Export");
                target.addEventListener("select", function(e) {

                    target = e.target;
                    if (!/\.csv$/.test(target.nativePath))
                        target.nativePath += ".csv";

                    file.writeText(target, csvText);
                });
            });

            $("#search-in-fiverr").click(function() {
                $("#filter-country").html('<option value="">Country</option>');
                for (var i in countries) {
                    var country = countries[i];
                    $("#filter-country").append('<option value="' + country.code + '">' + country.name + '</option>');
                }
            });

            $("#search-in-seoclerks").click(function() {
                $("#filter-country").html('<option value="">Country</option>');
                for (var i in countries) {
                    var country = countries[i];
                    $("#filter-country").append('<option value="' + country.code + '">' + country.code + '</option>');
                }
            });
            //EVENT HANDLERS
        });
    });