/// <reference path="/dev/vejis.intellisense.js" />
/// <reference path="sizzle.js" />

require_("script/vejis/dom.js");
require_("script/table.js");
require_("script/vejis/drop.js");
require_("script/vejis/drop.ui.js");

require_("script/http-request.js");
require_("script/file.js");
require_("script/restore.js");
require_("script/search/search.js");
require_("script/order/order.js");
require_("script/projects/projects.js");
require_("script/favorites/favorites.js");

use_("http-request", "file", "search", "order", "projects", "favorites", "dom", "enhanced-table", "drop", "drop.ui",
    function(hr, file, search, order, projects, favorites, dom, enhancedTable, drop) {
        module_("ui", function() {
            var ui = this;

            this.data = drop.createData({
                common: {
                    fiverr: {
                        username: undefined,
                        password: undefined
                    },
                    seoclerks: {
                        username: undefined,
                        password: undefined
                    },
                    currentTab: 0
                }
            });

            var common = ui.data("common");

            common.bind("proxiesText", function() {
                var text = this("proxiesText");
                if (typeof text != "string") return;

                var re = /([a-z0-9-\.]+)(?::(\d+))?/gi;
                var groups;

                hr.defaultProxies.length = 0;

                while (groups = re.exec(text)) {
                    hr.defaultProxies.push({
                        host: groups[1],
                        port: Number(groups[2]) || 80
                    });
                }
            });

            //this.resize = resize;

            var eles =
                this.eles = {};

            "main ui part",
            function() {
                "id to vars",
                function() {
                    for_(document.getElementsByTagName("*"), function(ele) {
                        var id = ele.id;
                        if (!id) return;
                        id = id.replace(/[-_]([a-z0-9])/ig, function(m, g1) {
                            return g1.toUpperCase();
                        });
                        eles[id] = ele;
                    });
                }();

                "initialize",
                function() {
                    var userFile = new air.File("app-storage:/user");
                    var user = file.readText(userFile);

                    if (document.location.href.indexOf("?restore") >= 0) {
                        use_("restore", function(restore) {
                            restore.restore();
                        });
                        if (user) {
                            userEmail = user;
                            setAds(user);
                        }
                        return;
                    }

                    /* DEBUG CODE
                dom.removeClass(eles.rootWrapper, "activiation");
                initNormalSize();
                return;
                //*/

                    var mac = "";
                    for_(air.NetworkInfo.networkInfo.findInterfaces(), function(info) {
                        if (info.hardwareAddress) {
                            var nmac = info.hardwareAddress.replace(/[^0-9a-z]/ig, "").toUpperCase();
                            nmac = nmac.match(/.{2}/g).join("-");
                            if (nmac != "00-00-00-00-00-00") {
                                mac = nmac;
                                return false;
                            }
                        }
                    });

                    var input = eles.licenseUserText;

                    input.onkeypress = function(e) {
                        if (e.keyCode != 13) return;

                        user = input.value;
                        if (!user) {
                            message.alert("Please enter a username.", function() {
                                input.focus();
                            });
                            return;
                        }

                        file.writeText(userFile, user);
                        input.disabled = true;

                        check(user);
                    };

                    if (!user) {
                        dom.hide(eles.licenseCheckingWrapper);
                        dom.show(eles.licenseUserWrapper);
                    } else {
                        check(user);
                    }

                    function check(user) {
                        dom.hide(eles.licenseUserWrapper);
                        dom.show(eles.licenseCheckingWrapper);

                        var url = "http://license.web1syndication.com/37JxDBkIczq04/gp/getUser.json/" + encodeURIComponent(user) + "/" + mac + "/";

                        var req = new air.URLRequest(url);
                        var loader = new air.URLLoader();
                        loader.load(req);

                        loader.addEventListener(air.Event.COMPLETE, function() {
                            var result;
                            try {
                                result = eval("(" + loader.data + ")");
                            } catch (e) {
                                result = {};
                            }
                            process(result);
                        });

                        loader.addEventListener(air.IOErrorEvent.IO_ERROR, function() {
                            process({});
                        });

                        function process(result) {
                            //result.result = "success";
                            if (result.result == "success"
                                //|| user == "i@vilic.info"
                            ) {
                                setAds(user);
                                userEmail = user;
                                dom.removeClass(eles.rootWrapper, "activiation");
                                initNormalSize();
                                ui.data("common.currentTab", 0);
                            } else {
                                var msg;
                                switch (result.reason) {
                                    case "ERROR_WHILE_INSERTING_USER":
                                    case "ERROR_WHILE_INSERTING_MAC":
                                    case "GET_USER_FAILED":
                                        msg = "Database error. Please contact us at: http://web1support.com";
                                        break;
                                    case "NO_OAP_TAG":
                                        msg = "You have not purchased this product. If you feel this is an error, submit a ticket at http://web1support.com";
                                        break;
                                    case "ACCOUNT_NOT_ACTIVE":
                                        msg = "Either you were banned, or account was disabled for another reason. If you feel this is an error, submit a ticket at http://web1support.com";
                                        break;
                                    case "MAC_LIMIT_REACHED":
                                        msg = "You have hit the MAC Address limit for your account. If you need to reset this, please submit a ticket at http://web1support.com";
                                        break;
                                    default:
                                        msg = "Unknown error. Please contact us at: http://web1support.com";
                                        break;
                                }
                                message.alert(msg, function() {
                                    input.disabled = false;
                                    user = input.value = "";
                                    file.writeText(userFile, user);
                                    dom.hide(eles.licenseCheckingWrapper);
                                    dom.show(eles.licenseUserWrapper);
                                });

                            }
                        }
                    }

                    function setAds(user) {
                        // Iframe Ad Interface
                        var air_interface = {};
                        air_interface.navigate = function(gourl) {
                            air.navigateToURL(new air.URLRequest(gourl));
                        };

                        //Pushing Ads
                        eles.adOne.src = "http://license.web1syndication.com/37JxDBkIczq04/gp/getAd1/" + user;
                        eles.adTwo.src = "http://license.web1syndication.com/37JxDBkIczq04/gp/getAd2/" + user;
                        eles.adOne.contentWindow.parentSandboxBridge = air_interface;
                        eles.adTwo.contentWindow.parentSandboxBridge = air_interface;
                    }
                }();

                "tabs",
                function() {
                    var tabs = dom.query("#main-tabs li");

                    var active = {
                        tab: null,
                        content: null
                    };

                    for_(tabs, function(tab, i) {
                        tab.addEventListener("click", function() {
                            ui.data("common.currentTab", i);
                        });
                        //if (i == 0) onclick();
                    });

                    ui.data.bind("common.currentTab", function() {
                        var index = this("common.currentTab");

                        var tab = tabs[index];
                        var content = document.getElementById(tab.getAttribute("data-bind"));

                        if (active.tab != tab) {
                            if (active.tab) {
                                dom.removeClass(active.tab, "active");
                                //if (active.content)
                                dom.hide(active.content);
                            }

                            active.tab = tab;
                            active.content = content;

                            dom.addClass(tab, "active");
                            dom.show(content);
                        }
                    });
                }();

                "common panel behavior",
                function() {
                    drop.ui.bindTextBox(common, "fiverr.username", eles.fiverrUsername);
                    drop.ui.bindTextBox(common, "fiverr.password", eles.fiverrPassword);
                    drop.ui.bindTextBox(common, "seoclerks.username", eles.seoclerksUsername);
                    drop.ui.bindTextBox(common, "seoclerks.password", eles.seoclerksPassword);

                    setTimeout(fiverrLogin, 1000);
                    $("#fiverr-username, #fiverr-password").blur(function() {
                        fiverrLogin();
                    });

                    setTimeout(seoclerksLogin, 1000);
                    $("#seoclerks-username, #seoclerks-password").blur(function() {
                        seoclerksLogin();
                    });

                    $("#fiverr-check-account").click(function() {
                        fiverrLogin(true);
                    });

                    $("#seoclerks-check-account").click(function() {
                        seoclerksLogin(true);
                    });

                    $("#overlay").click(function() {
                        $(this).hide();
                        if ($("#add-project-form").is(":visible")) {
                            $("#projects-cancel-btn").click();
                        }
                        if ($("#projects-gigs-wrapper").is(":visible")) {
                            projects.closeGigsPopup();
                        }
                    });

                    //setInterval(checkIframeSource, 1000);

                }(); //end of common panel behavior

                "search panel behavior",
                function() {
                    $("#search-in-fiverr, #search-in-seoclerks").click(function() {
                        $("#search-results-buttons-clear").click();
                    });

                    $("#search-add-project-btn").click(function() {
                        ui.data("common.currentTab", 3);
                        $("#projects-add-project-btn").click();
                        projects.init();
                    });

                    $("#search-in-fiverr").click(function() {
                        $('.fiverr-options').show();
                        $('.seoclerks-options').hide();
                    });

                    $("#search-in-seoclerks").click(function() {
                        $('.seoclerks-options').show();
                        $('.fiverr-options').hide();
                    });

                    $("#fiver-keyword").focus(function() {
                        if ($(this).val() == "Search...") $(this).val('');
                    });
                    $("#fiver-keyword").blur(function() {
                        if ($(this).val() == "") $(this).val('Search...');
                    });

                    for (var i in categories.fiverr) {
                        var category = categories.fiverr[i];
                        $("#fiverr-category").append('<option value="' + category.id + '">' + category.name + '</option>');
                    }

                    $("#fiverr-category").change(function() {
                        $("#fiver-keyword").val('Search...');
                        var category_id = $(this).val();
                        $("#fiverr-subcategory").select2("val", '0');
                        $("#fiverr-subcategory").html('<option value="0">Sub Category</option>');
                        for (var j in categories.fiverr[category_id].subcategories) {
                            var subcategory = categories.fiverr[category_id].subcategories[j];
                            $("#fiverr-subcategory").append('<option value="' + subcategory.id + '">' + subcategory.name + '</option>');
                        }
                    });

                    $("#fiverr-subcategory").change(function() {
                        $("#fiver-keyword").val('Search...');
                    });

                    $("#fiver-keyword").keyup(function() {
                        $("#fiverr-category").select2("val", '0');
                        $("#fiverr-subcategory").select2("val", '0');
                    });

                    $("#seoclerks-keyword").focus(function() {
                        if ($(this).val() == "Search...") $(this).val('');
                    });
                    $("#seoclerks-keyword").blur(function() {
                        if ($(this).val() == "") $(this).val('Search...');
                    });

                    for (var i in categories.seoclerks) {
                        var category = categories.seoclerks[i];
                        if (category.name != "Want to Buy" && category.name != "Want to Trade")
                            $("#seoclerks-category").append('<option value="' + category.url + '">' + category.name + '</option>');
                    }

                    $("#seoclerks-category").change(function() {
                        $("#seoclerks-keyword").val('Search...');
                        var category_url = $(this).val();
                        $("#seoclerks-subcategory").select2("val", '0');
                        $("#seoclerks-subcategory").html('<option value="0">Sub Category</option>');
                        for (var j in categories.seoclerks[category_url].subcategories) {
                            var subcategory = categories.seoclerks[category_url].subcategories[j];
                            $("#seoclerks-subcategory").append('<option value="' + subcategory.url + '">' + subcategory.name + '</option>');
                        }
                    });

                    $("#seoclerks-subcategory").change(function() {
                        $("#seoclerks-keyword").val('Search...');
                    });

                    $("#seoclerks-keyword").keyup(function() {
                        $("#seoclerks-category").select2("val", '0');
                        $("#seoclerks-subcategory").select2("val", '0');
                    });

                    $("#search-results-buttons-order").click(function() {
                        var asr = search.addSelectedResultsToOrder();

                        if (asr) {
                            ui.data("common.currentTab", 1);
                        }
                    });

                    for (var i in countries) {
                        var country = countries[i];
                        $("#filter-country").append('<option value="' + country.code + '">' + country.name + '</option>');
                    }

                    $(".search-filter").change(function() {
                        var search_type = $("#search-in-fiverr").is(":checked") ? "fiverr" : "seoclerks";
                        search.applyFilters(search_type);
                    });

                    $("#search-btn").click(function() {
                        search.init("fiverr");
                    });

                    $("#search2-btn").click(function() {
                        search.init("seoclerks");
                    });

                    $("#reset-filters-btn").click(function() {
                        $('#filter-delivery-time').select2("val", '');
                        $('#filter-positive-ratio').select2("val", '');
                        $('#filter-positive-ratings').select2("val", '');
                        $('#filter-country').select2("val", '');
                        search.applyFilters();
                    });

                    $("#search-results-buttons-save-to-favorites-gigs").click(function() {
                        search.saveSelectedResultsToFavorites();
                    });
                }(); //end of search panel behavior

                "order panel behavior",
                function() {}(); //end of order panel behavior

                "favorites panel behavior",
                function() {
                    favorites.init();
                    favorites.buildSelectsWithTabs();

                    $("#favorites-tab").click(function() {
                        var first_li = $("#favorites-tabs ul li:first");
                        var first_tab_id = first_li.attr("tab-id");
                        favorites.initTable(first_tab_id);
                        if ($("#favorite-gigs").is(':visible')) {
                            favorites.showFavoriteGigs();
                        }
                    });

                    $("#favorites-filter-country").html('<option value="">Country</option><option value="" disabled="disabled">Please select a site first</option>');

                    $("#favorites-site").change(function() {
                        $("#favorites-category").select2("val", '0');
                        $("#favorites-subcategory").select2("val", '0');
                        $("#favorites-filter-country").select2("val", '');

                        if ($(this).val() == "0") {
                            $("#favorites-category").html('<option value="0">Category</option>');
                            $("#favorites-subcategory").html('<option value="0">Sub Category</option>');
                            $("#favorites-filter-country").html('<option value="">Country</option>');
                        } else if ($(this).val() == "fiverr") {
                            $("#favorites-category").html('<option value="0">Category</option>');
                            for (var i in categories.fiverr) {
                                var category = categories.fiverr[i];
                                $("#favorites-category").append('<option value="' + category.id + '">' + category.name + '</option>');
                            }

                            $("#favorites-subcategory").html('<option value="0">Sub Category</option>');

                            $("#favorites-filter-country").html('<option value="">Country</option>');
                            for (var i in countries) {
                                var country = countries[i];
                                $("#favorites-filter-country").append('<option value="' + country.code + '">' + country.name + '</option>');
                            }
                        } else if ($(this).val() == "seoclerks") {
                            $("#favorites-category").html('<option value="0">Category</option>');
                            for (var i in categories.seoclerks) {
                                var category = categories.seoclerks[i];
                                if (category.name != "Want to Buy" && category.name != "Want to Trade")
                                    $("#favorites-category").append('<option value="' + category.url + '">' + category.name + '</option>');
                            }

                            $("#favorites-subcategory").html('<option value="0">Sub Category</option>');

                            $("#favorites-filter-country").html('<option value="">Country</option>');
                            for (var i in countries) {
                                var country = countries[i];
                                $("#favorites-filter-country").append('<option value="' + country.code + '">' + country.code + '</option>');
                            }
                        }
                    });

                    $("#favorites-keyword").focus(function() {
                        if ($(this).val() == "Keyword...") $(this).val('');
                    });
                    $("#favorites-keyword").blur(function() {
                        if ($(this).val() == "") $(this).val('Keyword...');
                    });

                    $("#favorites-keyword").keyup(function() {
                        $("#favorites-category").select2("val", '0');
                        $("#favorites-subcategory").html('<option value="0">Sub Category</option>');
                    });

                    $("#favorites-category").change(function() {
                        $("#favorites-keyword").val('Keyword...');

                        if ($("#favorites-site").val() == "fiverr") {
                            var category_id = $(this).val();
                            $("#favorites-subcategory").html('<option value="0">Sub Category</option>');
                            $("#favorites-subcategory").select2("val", 0);

                            if (category_id != "0") {
                                for (var j in categories.fiverr[category_id].subcategories) {
                                    var subcategory = categories.fiverr[category_id].subcategories[j];
                                    $("#favorites-subcategory").append('<option value="' + subcategory.id + '">' + subcategory.name + '</option>');
                                }
                            }
                        }

                        if ($("#favorites-site").val() == "seoclerks") {
                            var category_url = $(this).val();
                            $("#favorites-subcategory").html('<option value="0">Sub Category</option>');
                            $("#favorites-subcategory").select2("val", 0);

                            if (category_url != "0") {
                                for (var j in categories.seoclerks[category_url].subcategories) {
                                    var subcategory = categories.seoclerks[category_url].subcategories[j];
                                    $("#favorites-subcategory").append('<option value="' + subcategory.url + '">' + subcategory.name + '</option>');
                                }
                            }
                        }
                    });

                    $("#favorites-add-favorite").click(function() {

                        var fav_keyword = "";
                        var tab_id = null;

                        var data_type = "keyword";
                        var site = $("#favorites-site").val();

                        if (IsEmpty(site) || site == "0") {
                            message.alert("Please select a site.");
                            return false;
                        }

                        var keyword = $("#favorites-keyword").val();
                        var category = $("#favorites-category").val();
                        var subcategory = $("#favorites-subcategory").val();
                        var category_text = $("#favorites-category option:selected").text();
                        var subcategory_text = $("#favorites-subcategory option:selected").text();

                        if (!IsEmpty(keyword) && keyword != "Keyword...") {
                            fav_keyword = keyword;
                        }

                        if (category != 0) {
                            fav_keyword = category_text;
                            if (subcategory != 0) {
                                fav_keyword = category_text + " - " + subcategory_text;
                            }
                            var data_type = "category";
                        }

                        if (IsEmpty(fav_keyword)) {
                            message.alert("Please enter a keyword or select a category.");
                            return false;
                        }

                        var filters = {
                            delivery_time: {
                                value: $("#favorites-filter-delivery-time").val(),
                                text: $('#favorites-filter-delivery-time option:selected').text()
                            },
                            positive_ratio: {
                                value: $("#favorites-filter-positive-ratio").val(),
                                text: $('#favorites-filter-positive-ratio option:selected').text()
                            },
                            positive_ratings: {
                                value: $("#favorites-filter-positive-ratings").val(),
                                text: $('#favorites-filter-positive-ratings option:selected').text()
                            },
                            country: {
                                value: $("#favorites-filter-country").val(),
                                text: $('#favorites-filter-country option:selected').text()
                            }
                        };

                        favorites.addFavoriteToList(site, fav_keyword, data_type, tab_id, filters);
                        message.alert("Added to the list!");

                        $("#favorites-site").select2("val", "0");
                        $("#favorites-keyword").val("Keyword...");
                        $("#favorites-category").select2("val", "0");
                        $("#favorites-subcategory").select2("val", "0");
                        $("#favorites-category").html('<option value="0">Category</option>');
                        $("#favorites-subcategory").html('<option value="0">Sub Category</option>');
                        $("#favorites-filter-delivery-time").select2("val", "");
                        $("#favorites-filter-positive-ratio").select2("val", "");
                        $("#favorites-filter-positive-ratings").select2("val", "");
                        $("#favorites-filter-country").select2("val", "");
                        $("#favorites-filter-country").html('<option value="">Country</option>');

                        favorites.refreshTabs();

                        //favorites.addFavoriteToList($("#favorites-site").val(), $("#favorites-keyword").val());
                    });

                    $("#search-results-buttons-save-to-favorites").click(function() {
                        $(this).hide();
                        $("#search-results-save-to-favorites-area").show();
                    });

                    $("#search-results-buttons-save-to-favorites-cancel").click(function() {
                        $("#search-results-buttons-save-to-favorites").show();
                        $("#search-results-save-to-favorites-area").hide();
                    });

                    $("#search-results-buttons-save-to-favorites-save").click(function() {
                        var fav_keyword = "";
                        var tab_id = $("#search-select-tab").val();

                        var data_type = "keyword";
                        var site = $("#search-in-fiverr").is(":checked") ? "fiverr" : "seoclerks";;

                        if (site == "fiverr") {
                            var keyword = $("#fiver-keyword").val();
                            var category = $("#fiverr-category").val();
                            var subcategory = $("#fiverr-subcategory").val();
                            var category_text = $("#fiverr-category option:selected").text();
                            var subcategory_text = $("#fiverr-subcategory option:selected").text();

                        } else if (site == "seoclerks") {
                            var keyword = $("#seoclerks-keyword").val();
                            var category = $("#seoclerks-category").val();
                            var subcategory = $("#seoclerks-subcategory").val();
                            var category_text = $("#seoclerks-category option:selected").text();
                            var subcategory_text = $("#seoclerks-subcategory option:selected").text();
                        }

                        if (!IsEmpty(keyword) && keyword != "Search...") {
                            fav_keyword = keyword;
                        }

                        if (category != 0) {
                            fav_keyword = category_text;
                            if (subcategory != 0) {
                                fav_keyword = category_text + " - " + subcategory_text;
                            }
                            var data_type = "category";
                        }

                        if (IsEmpty(fav_keyword)) {
                            message.alert("Please enter a keyword or select a category.");
                            return false;
                        }

                        var filters = {
                            delivery_time: {
                                value: $("#filter-delivery-time").val(),
                                text: $('#filter-delivery-time option:selected').text()
                            },
                            positive_ratio: {
                                value: $("#filter-positive-ratio").val(),
                                text: $('#filter-positive-ratio option:selected').text()
                            },
                            positive_ratings: {
                                value: $("#filter-positive-ratings").val(),
                                text: $('#filter-positive-ratings option:selected').text()
                            },
                            country: {
                                value: $("#filter-country").val(),
                                text: $('#filter-country option:selected').text()
                            }
                        };

                        favorites.addFavoriteToList(site, fav_keyword, data_type, tab_id, filters);
                        message.alert("Added to favorites!");
                        $("#search-results-buttons-save-to-favorites-cancel").click();
                    });

                    $("#view-orders-btn").click(function() {
                        projects.showOrdersForProject();
                    });

                    $("#project-details-btn").click(function() {
                        projects.showProjectDetails();
                    });
                }(); //end of favorites panel behavior

                "projects panel behavior",
                function() {
                    projects.buildSelectsWithProjects();
                    $("#projects-tab").click(function() {
                        projects.init();
                    });
                    $("#projects-cancel-btn").click(function() {
                        $("#add-project-form input, #add-project-form textarea").val('');
                        $("#add-project-form").hide();
                    });
                    $("#projects-save-btn").click(function() {
                        projects.saveProject();
                    });
                    $("#projects-delete-projects-btn").click(function() {
                        projects.deleteProjects();
                    });
                }(); //end of sites panel behavior

                var spam_answer = "";

                function fiverrLogin(is_check) {
                    if (!is_check && $("#captcha-answer-wrapper").is(':visible')) {
                        return false;
                    }

                    var username = $("#fiverr-username").val();
                    var password = $("#fiverr-password").val();
                    if (is_check) {
                        $("#fiverr-check-account").text("checking...");
                        if (IsEmpty(username) || IsEmpty(password)) {
                            message.alert('Please fill in username and password.');
                            $("#fiverr-check-account").text("check");
                            return false;
                        }
                    } else {
                        if (IsEmpty(username) || IsEmpty(password)) {
                            return false;
                        }
                    }

                    if (!$("#captcha-answer-wrapper").is(':visible')) {
                        var req2 = new air.URLRequest("http://fiverr.com/logout");
                        var loader2 = new air.URLLoader();
                        loader2.load(req2);

                        loader2.addEventListener(air.Event.COMPLETE, function() {
                            login();
                        });
                    }
                    else {
                        air.trace('without logout');
                        login();
                    }

                    function login() {
                        var url = "http://fiverr.com/user_sessions";
                        var query = new hr.FormData();

                        var req = new air.URLRequest(url);

                        var data = new air.URLVariables();
                        data["user_session[login]"] = username;
                        data["user_session[password]"] = password;

                        if ($("#captcha-answer-wrapper").is(':visible')) {
                            data["user[spam_answers]"] = spam_answer;
                            data["user[spam_answer]"] = $("#captcha-answer").val();
                            data["user[failed_login_count]"] = "2";
                        }

                        req.method = "POST";
                        req.data = data;

                        var loader = new air.URLLoader();

                        loader.load(req);
                        loader.addEventListener("complete", function() {
                            //air.trace(loader.data);
                            var html = loader.data;

                            var splitted = html.split("user[spam_answer]");
                            air.trace(splitted.length);

                            if (splitted.length > 1) {
                                ui.data("common.currentTab", 4);
                                var captcha_text = getValueBetweenStrings(html, '<label for="user_spam_answer">', '</label>');
                                $("#captcha-answer-wrapper").show();
                                if (IsEmpty(captcha_text)) {
                                    $("#captcha-answer-wrapper").hide();
                                    $('span[for="captcha-answer"]').text('');
                                    $("#captcha-answer").val('');
                                    spam_answer = "";

                                    if (is_check) {
                                        message.alert('Wrong username or password.');
                                    }
                                }
                                else {
                                    $('span[for="captcha-answer"]').text(captcha_text);
                                    $("#captcha-answer").val('');
                                    $("#captcha-answer").focus();

                                    spam_answer = getValueBetweenStrings(html, 'name="user[spam_answers]" type="hidden" value="', '"');
                                }

                            } else {
                                $("#captcha-answer-wrapper").hide();
                                $('span[for="captcha-answer"]').text('');
                                $("#captcha-answer").val('');
                                spam_answer = "";
                            }

                            if (is_check) {
                                if (splitted.length == 1) {
                                    var html = loader.data;
                                    var splitted = html.split(username);

                                    if (splitted.length > 5) {
                                        message.alert('Success! Account is valid.');
                                    } else {
                                        message.alert('Account is NOT valid!');
                                    }
                                }
                            }


                            //fiverrLogin();
                        });

                        loader.addEventListener("httpStatus", function(e) {
                            //air.trace(e);
                            if (is_check) $("#fiverr-check-account").text("check");
                        });
                        //$("#fiverr-login-username").val($("#fiverr-username").val());
                        //$("#fiverr-login-password").val($("#fiverr-password").val());
                        //document.fiverrLoginForm.submit();
                    }
                }

                function seoclerksLogin(is_check) {
                    var username = $("#seoclerks-username").val();
                    var password = $("#seoclerks-password").val();
                    if (is_check) {
                        $("#seoclerks-check-account").text("checking...");
                        if (IsEmpty(username) || IsEmpty(password)) {
                            message.alert('Please fill in username and password.');
                            $("#seoclerks-check-account").text("check");
                            return false;
                        }
                    } else {
                        if (IsEmpty(username) || IsEmpty(password)) {
                            return false;
                        }
                    }

                    var url = "https://www.seoclerks.com/login";
                    var query = new hr.FormData();

                    var req = new air.URLRequest(url);

                    var data = new air.URLVariables();
                    data["l_username"] = username;
                    data["l_password"] = password;
                    data["r"] = "";
                    data["jlog"] = "1";

                    req.method = "POST";
                    req.data = data;

                    var loader = new air.URLLoader();

                    loader.load(req);
                    loader.addEventListener("complete", function() {
                        //air.trace(loader.data);
                        if (is_check) {
                            var html = loader.data;
                            var splitted = html.split(username);

                            if (splitted.length > 5) {
                                message.alert('Success! Account is valid.');
                            } else {
                                message.alert('Account is NOT valid!');
                            }
                        }
                    });

                    loader.addEventListener("httpStatus", function(e) {
                        if (is_check) $("#seoclerks-check-account").text("check");
                        //air.trace(e);
                    });

                    //$("#seoclerks-login-username").val($("#seoclerks-username").val());
                    //$("#seoclerks-login-password").val($("#seoclerks-password").val());
                    //document.seoclerksLoginForm.submit();
                }

                var proxiesFile = new air.File("app-storage:/proxies");
                var savedProxiesTxt = file.readText(proxiesFile);
                var savedProxies = IsEmpty(savedProxiesTxt) ? [] : $.parseJSON(savedProxiesTxt);

                var proxiesTxt = "";

                for (var i in savedProxies) {
                    proxiesTxt = proxiesTxt + savedProxies[i].proxy + "\n";
                }

                $("#proxiesTxt").val(proxiesTxt);

                $("#proxiesTxt").keyup(function() {
                    var proxiesFile = new air.File("app-storage:/proxies");
                    var savedProxiesTxt = file.readText(proxiesFile);
                    var savedProxies = IsEmpty(savedProxiesTxt) ? [] : $.parseJSON(savedProxiesTxt);
                    air.trace(savedProxiesTxt);
                    var proxies = [];

                    var exploded = $(this).val().split("\n");

                    for (var i in exploded) {
                        var this_proxy = exploded[i];
                        var existsAlready = false;
                        for (var k in savedProxies) {
                            if (savedProxies[k].proxy == this_proxy) {
                                existsAlready = true;
                                var existingProxy = savedProxies[k];
                            }
                        }

                        if (existsAlready)
                            proxies.push(existingProxy);
                        else {
                            if (!IsEmpty(this_proxy))
                                proxies.push({
                                    proxy: this_proxy
                                });
                        }
                    }

                    air.trace(JSON.stringify(proxies));

                    file.writeText(proxiesFile, JSON.stringify(proxies));
                });

                $("#proxies-check").click(function() {
                    var proxiesFile = new air.File("app-storage:/proxies");
                    var savedProxiesTxt = file.readText(proxiesFile);
                    var savedProxies = IsEmpty(savedProxiesTxt) ? [] : $.parseJSON(savedProxiesTxt);

                    if (savedProxies.length == 0) {
                        message.alert("Please enter some proxies first");
                        return false;
                    }

                    $(this).html("checking...processed <strong id='processed_proxies_nr'>0</strong> out of <strong>" + savedProxies.length + "</strong> proxies");

                    var proxies = [];

                    var good_fiverr = 0;
                    var good_seoclerks = 0;
                    var proxy_counter = 0;
                    processProxy();

                    function processProxy() {
                        if (!savedProxies[proxy_counter]) {
                            $("#proxies-check").text('check');

                            var proxiesTxt = "";

                            for (var i in proxies) {
                                proxiesTxt = proxiesTxt + proxies[i].proxy + "\n";
                            }

                            $("#proxiesTxt").val(proxiesTxt);
                            file.writeText(proxiesFile, JSON.stringify(proxies));

                            message.alert('Testing proxies finished. There are <strong>' + good_fiverr + '</strong> good proxies. Bad proxies have been removed.');
                            return false;
                        }

                        var proxy = savedProxies[proxy_counter];

                        var req = new hr.Request();

                        var url = "http://fiverr.com/gigs/gigs_as_json?host=search&type=best_match&query_string=seo&limit=10&page=1";

                        var api_url = url;

                        var exploded = proxy.proxy.split(":");
                        req.defaultProxies = [{
                            host: exploded[0],
                            port: exploded[1]
                        }];

                        req.open("get", api_url);
                        req.send(function() {
                            var thisGoodFiverr = false;
                            if (this.status == 200) {
                                var html = this.responseText;
                                this.dispose();

                                if (!IsEmpty(html)) {
                                    var results = $.parseJSON(html);

                                    if (results != null) {
                                        if (results.gigs) {
                                            var gigs = results.gigs;
                                            if (gigs.length > 0) {
                                                thisGoodFiverr = true;
                                                good_fiverr++;
                                            }
                                        }
                                    }
                                }
                            }

                            savedProxies[proxy_counter].fiverr = thisGoodFiverr;

                            if (savedProxies[proxy_counter].fiverr) {
                                proxies.push(savedProxies[proxy_counter]);
                                proxy_counter++;
                                $("#processed_proxies_nr").text(proxy_counter);
                                processProxy();
                            }
                        });
                    }
                });


                function checkIframeSource() {
                    var iframe = document.getElementById("gigOrderFrame");
                    air.trace(iframe.src);
                }

            }();

            document.location.href.indexOf("?restore") < 0 &&
                function() {
                    var win = nativeWindow;
                    var screen = air.Screen.mainScreen;
                    win.x = (screen.bounds.width - win.width) / 2;
                    win.y = (screen.bounds.height - win.height) / 2;
                    win.visible = true;
            }();
        });
    });