﻿/// <reference path="/dev/vejis.intellisense.js" />

require_("script/vejis/drop.js");
require_("script/vejis/dom.js");

use_("drop", "dom", function (drop, dom) {
    module_("drop.ui", function () {
        var ui = drop.ui = this;

        this.bindTextBox = _(Object, String, Object, opt_(nul_(Object)), opt_(Boolean, false),
        function (data, name, textBox, checker, autoChangeBack) {
            var hasChecker = checker != undefined;

            data.bind(name, function () {
                if (!setAsDataValue()) {
                    if (checkValue()) {
                        //use the textBox's a preplaced value.
                        setDataValue();
                    }
                }
            });

            dom.on(textBox, "blur", function () {
                if (checkValue()) {
                    setDataValue();
                    dom.removeClass(textBox, "invalid");
                }
                else
                    dom.addClass(textBox, "invalid");
            });

            function checkValue() {
                var correct = (
                    !hasChecker ||
                    (typeof checker == "function" && checker(textBox.value)) ||
                    (checker instanceof RegExp && checker.test(textBox.value))
                );

                if (!correct && autoChangeBack) {
                    setAsDataValue();
                    correct = true;
                }

                return correct;
            }

            function setAsDataValue() {
                var value = data(name);
                if (value != null) {
                    value = value.toString();
                    if (textBox.value != value) {
                        textBox.value = value;
                        dom.fireEvent(textBox, "change");
                    }
                    dom.removeClass(textBox, "invalid");
                    return true;
                }
                return false;
            }

            function setDataValue() {
                var value = data(name);

                var nValue = textBox.value;;
                if (typeof value == "number")
                    nValue = Number(nValue);

                data(name, nValue);
            }
        });

        this.bindCheckBox = _(Object, String, Object, function (data, name, checkBox) {
            data.bind(name, function () {
                var checked = this(name);
                if (typeof checked == "boolean") {
                    checkBox.checked = checked;
                    dom.fireEvent(checkBox, "change");
                }
                else
                    this(name, !!checkBox.checked);
            });

            dom.on(checkBox, "change", function () {
                data(name, checkBox.checked);
            });
        });

        this.bindRadios = _(Object, String, IList, function (data, name, radios) {
            var map = {};

            var initValue;

            for_(radios, function (radio) {
                var value = radio.value;

                if (!value) return;

                map[value] = radio;
                if (radio.checked)
                    initValue = value;

                dom.on(radio, "change", function () {
                    if (radio.checked)
                        data(name, value);
                });
            });

            data.bind(name, function () {
                var value = this(name);

                if (typeof value != "string" && typeof initValue == "string") {
                    this(name, initValue);
                }
                else {
                    var radio = map[value];
                    if (radio && !radio.checked) {
                        radio.checked = true;
                        dom.fireEvent(radio, "change");
                    }
                }
            });

        });

        this.bindInnerHTML = _(Object, String, Object, opt_(String), opt_(Boolean, false), function (data, name, ele, defaultValue, noEscape) {

            data.bind(name, function () {
                var value = this(name);

                if (value == null)
                    value = defaultValue || "";
                else
                    value = value.toString();

                if (!noEscape)
                    value = value.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/\r?\n/g, "<br />");
                ele.innerHTML = value;
            });

        });

        this.bindDisplay = _(Object, String, Object, opt_(nul_(Function)), function (data, name, ele, checker) {
            data.bind(name, function () {
                var value = this(name);

                var result;
                if (checker) {
                    result = checker(value);
                }

                if (typeof result == "undefined")
                    result = !!value;

                (result ? dom.show : dom.hide)(ele);
            });
        });

        this.bindDisable = _(Object, String, Object, opt_(nul_(Function)), function (data, name, ele, checker) {
            data.bind(name, function () {
                var value = this(name);

                var result;
                if (checker) {
                    result = checker(value);
                }

                if (typeof result == "undefined")
                    result = !!value;

                ele.disabled = result;
            });
        });
    });
});